FALSE			EQU				0
TRUE			EQU				NOT(FALSE)

;------------------------------------------------------------------------------------------------------------
;hardware register definitions
;------------------------------------------------------------------------------------------------------------

PORT1DDR		EQU				$00							;Port 1 data direction register
PORT2DDR		EQU				$01							;Port 2 data direction register
PORT1			EQU				$02							;Port 1
PORT2			EQU				$03							;Port 2

PORT3DDR		EQU				$04							;Port 3 data direction register
PORT4DDR		EQU				$05							;Port 4 data direction register
PORT3			EQU				$06							;Port 3
PORT4			EQU				$07							;Port 4
TCSR1			EQU				$08							;Timer control/status register 1
FRCOUNTER		EQU				$09							;16 bit free running counter
OCR1			EQU				$0B							;16 bit output compare register 1
ICR				EQU				$0D							;16 bit input compare register
TCSR2			EQU				$0F							;Timer control/status register 2
RMCR			EQU				$10							;Rate, mode control register
TRCSR1			EQU				$11							;Tx/Rx control status register 1
RDR				EQU				$12							;Receive data register
TDR				EQU				$13							;Transmit data register
RP5CR			EQU				$14							;RAM/port 5 control
PORT5			EQU				$15							;Port 5
PORT6DDR		EQU				$16							;Port 6 data direction register
PORT6			EQU				$17							;Port 6
PORT7			EQU				$18							;Port 7
OCR2			EQU				$19							;16 bit output compare register 2
TCSR3			EQU				$1B							;Timer control/status register 3
TCONR			EQU				$1C							;Timer constant register
T2CNT			EQU				$1D							;Timer 2 Upcounter register
TRCSR2			EQU				$1E							;Tx/Rx control status register 2
TESTREG			EQU				$1F
PORT5DDR		EQU				$20							;Port 5 data direction register
P6CSR			EQU				$21							;Port 6 control/status register


;
;Equates for timer control/status register 1 (TCSR1)
;
OLVL1			EQU				$01							;Output level 1
IEDG			EQU				$02							;Input edge
ETOI			EQU				$04							;Enable timer overflow interrupt
EOCL1			EQU				$08							;Enable output compare interrupt 1
EICI			EQU				$10							;Enable input compare interrupt 
TOF				EQU				$20							;Timer overflow flag
ICF				EQU				$80							;Input compare flag


;
;Equates for timer control/status register 2 (TCSR2)
;
OE1				EQU				$01							;Output enable 1
OE2				EQU				$02							;Output enable 2
OLVL2			EQU				$04							;Output level 2
EOCI2			EQU				$08							;Enable output compare interrupt 2

OCF2			EQU				$20							;Output compare flag 2
OCF1			EQU				$40							;Output compare flag 1

;
;Equates for timer control/status register 3 (TCSR3)
;
CKS0			EQU				$01							;Input clock select 0
CKS1			EQU				$02							;Input clock select 2
TOS0			EQU				$04							;Timer output select 0
TOS1			EQU				$08							;Timer output select 1
T2E				EQU				$10							;Timer 2 enable
ECMI			EQU				$40							;Enable timer match interrupt
CMF				EQU				$80							;Counter match flag

;
;Equates for Ram/port 5 control register (RP5CR)
;
IRQ1E			EQU				$01							;IRQ 1 enable
IRQ2E			EQU				$02							;IRQ 2 enable
MRE				EQU				$04							;MR (memory ready) enable
HALTE			EQU				$08							;HALT enabled
AMRE			EQU				$10							;Auto memory ready enable
STBYFLAG		EQU				$20							;Standby flag
RAME			EQU				$40							;RAM enable
STBYPWR			EQU				$80							;Standby power

;
;Equates for Port 6 control/status register (P6CSR)
;
P6LE			EQU				$08							;Port 6 latch enable
OSS				EQU				$10							;Output strobe select
OSE				EQU				$20							;Output strobe enable
ISIRQE			EQU				$40							;Input strobe interrupt enable
ISFLAG			EQU				$80							;Input strobe flag


;
;Command/status data bus out from the emulator
;
CTRL_REG0		EQU				$4000

CR0_mDATA		EQU				$0F							;Command/status nibble
CR0_iDATA		EQU				$00
CR0_aDATA		EQU				$0F

CR0_mIRQ		EQU				$10							;Set this active to start command/status
CR0_iIRQ		EQU				$00							;sequence
CR0_aIRQ		EQU				$10

CR0_mCDCK		EQU				$20							;Command/status handshaking line out from
CR0_iCDCK		EQU				$00							;the emulator
CR0_aCDCK		EQU				$20

;
;Command/status read equates
;
STAT_REG0		EQU				$4000
SR0_mDATA		EQU				$0F							;Command/status nybble

SR0_mHSCK		EQU				$10							;Command/status handshaking line
SR0_iHSCK		EQU				$10							;in to the emulator
SR0_aHSCK		EQU				$00

SR0_mRESET		EQU				$20							;The state of the reset line to
SR0_iRESET		EQU				$20							;the emulator
SR0_aRESET		EQU				$00

;
;Misc. audio control lines and LED!
;

CTRL_REG1		EQU				$4001

CR1_mDOUT		EQU				$01							;Tells if rom or music is playing
CR1_iDOUT		EQU				$01
CR1_aDOUT		EQU				$00

CR1_mEMP		EQU				$02							;If audio emphasis is on
CR1_iEMP		EQU				$02
CR1_aEMP		EQU				$00

CR1_mMUTE		EQU				$04							;If active muting is on
CR1_iMUTE		EQU				$00
CR1_aMUTE		EQU				$04

CR1_mC2PO		EQU				$08							;this is the error flag (not normally used during emulation)
CR1_iC2PO		EQU				$00
CR1_aC2PO		EQU				$08

CR1_mLED		EQU				$80							;It's the great and powerful LED of OZ.
CR1_iLED		EQU				$80
CR1_aLED		EQU				$00


;
;Port5 better known as CTRL_REG2 defines
;
CTRL_REG2		EQU				PORT5						;Port 5 is control register 2

CR2_mBANKIRQ	EQU				$04							;Set this inactive to clear the bank IRQ.
CR2_iBANKIRQ	EQU				$00							;Inactive will disable bank IRQ.
CR2_aBANKIRQ	EQU				$04

CR2_mH2ISTROBE  EQU				$20							;strobe to IBM
CR2_iH2ISTROBE  EQU				$20
CR2_aH2ISTROBE  EQU				$00

CR2_mH2ISHAKE   EQU				$40							;Handshake line to the IBM	
CR2_iH2ISHAKE   EQU				$00
CR2_aH2ISHAKE   EQU				$40


STAT_REG1		EQU				PORT5

SR1_mBANKIRQ	EQU				$01							;If active bank IRQ happend
SR1_iBANKIRQ	EQU				$01
SR1_aBANKIRQ	EQU				$00

SR1_mI2HSTROBE  EQU				$10
SR1_iI2HSTROBE  EQU				$00
SR1_aI2HSTROBE  EQU				$10

SR1_mI2HSHAKE   EQU				$80							;input handshake line from the IBM
SR1_iI2HSHAKE   EQU				$00
SR1_aI2HSHAKE   EQU				$80


I2HDATA			EQU				PORT2						;IBM to host data lines
H2IDATA			EQU				PORT6						;Host to IBM data lines


INTERNALRAMSTART EQU			$0040						;Where the internal RAM starts
INTERNALRAMEND  EQU				$013F						;Where it ends

EXTERNALRAMSTART EQU			$8000						;Start of the code ram
EXTERNALRAMEND  EQU				$FFFF

CYCLESPERMS		EQU				2117						;number of processor cycles in 1 ms

;various timeout values

IRQTIMEOUT		EQU				2							;number of ms before HSCK required after IRQ
HSCKTIMEOUT		EQU				3							;number of ms to complete status/command sequence
H2ITIMEOUT		EQU				3							;number of ms to wait for handshake from the IBM
SCANPLAYTIME	EQU				10							;while scanning, play at least this many blocks before seeking again

;constants relating to emulation

TOCTVERSION		EQU				0							;version number to report to a TOCT request

ADJUSTBLOCKBACK	EQU				4							;number of blocks to subtract from seek request to guarantee that the requested block is seen

FWDOFFSETLOW	EQU				100							;low word of number of blocks to skip while in forward scan mode
FWDOFFSETHIGH	EQU				0							;high word of number of blocks to skip while in forward scan mode

RVSOFFSETLOW	EQU				140							;low word of number of blocks to skip while in reverse scan mode
RVSOFFSETHIGH	EQU				0							;high word of number of blocks to skip while in reverse scan mode

SB_STATUS		EQU				0							;offset into the status buffer where the status is
SB_REQCD		EQU				1							;request type
SB_MIN			EQU				2							;minutes for A, R, TOCO, and TOCN
SB_SEC			EQU				4							;seconds for A, R, TOCO, and TOCN
SB_FRAME		EQU				6							;frames for A, R, TOCO, and TOCN
SB_FLAGS		EQU				8							;flags for all but TOCN
SB_TNO			EQU				2							;track number for T
SB_CTRL			EQU				6							;CTRL for T
SB_FIRSTTNO		EQU				2							;first track number for TOCT
SB_LASTTNO		EQU				4							;last track number for TOCT
SB_VERSION		EQU				6							;version for TOCT
SB_N			EQU				8							;N for TOCN
SB_ERRORNUMBER	EQU				2							;error number for E
SB_SUM			EQU				9							;offset into the status buffer where the checksum is

CB_COMMAND0		EQU				0							;0th nibble of command
CB_COMMAND1		EQU				1							;1st nibble of command
CB_REQTYPE		EQU				3							;location of the request type
CB_TOCNTNO		EQU				4							;track number for TOCN request command
CB_MIN			EQU				2							;time
CB_SEC			EQU				4
CB_FRAME		EQU				6
CB_DIRECTION	EQU				3							;track skip direction
CB_NUMTRACKS	EQU				4							;number of "tracks" to skip
CB_CUETNO		EQU				2							;track number for track cue command
CB_SUM			EQU				9							;offset into the command buffer where the checksum is

STATUSBUFFERLENGTH EQU			10							;number of nybbles in status buffer
COMMANDBUFFERLENGTH EQU			10							;number of nybbles in command buffer

;drive command nibbles

CMD_NOP			EQU				$0							;nop command
CMD_STOP		EQU				$1							;stop motor
CMD_REPORTREQ	EQU				$2							;change report type
CMD_READ		EQU				$3							;read ROM data
CMD_SEEK		EQU				$4							;seek to a given location
CMD_PAUSE		EQU				$6							;pause the drive
CMD_PLAY		EQU				$7							;start playing from current location
CMD_FWD			EQU				$8							;forward skip and playback
CMD_RVS			EQU				$9							;reverse skip and playback
CMD_TRACKSKIP	EQU				$A							;start track skipping
CMD_TRACKCUE	EQU				$B							;track cueing
CMD_DOORCLOSE	EQU				$C							;close the door
CMD_DOOROPEN	EQU				$D							;open the door

;drive status nibbles

STATUS_STOP		EQU				$0							;disc is set, motor not in operation
STATUS_PLAY		EQU				$1							;data or audio playback in progress
STATUS_SEEK		EQU				$2							;seek operation by read or seek command
STATUS_SCAN		EQU				$3							;skipping to a specific track
STATUS_PAUSE	EQU				$4							;paused at designated track or frame
STATUS_DOOROPEN	EQU				$5							;the door mechanism is open
STATUS_SUMERROR	EQU				$6							;a checksum error occurred during communication
STATUS_COMMANDERROR	EQU			$7							;an invalid command has arrived
STATUS_FUNCTIONERROR	EQU		$8							;some sort of error occurred during command execution
STATUS_TOCREAD	EQU				$9							;TOC is being read at the moment
STATUS_TRACKING	EQU				$A							;currently skipping tracks
STATUS_NODISC	EQU				$B							;door is closed, but cannot focus on disc
STATUS_DISCEND	EQU				$C							;paused in the leadout
STATUS_DISCIN	EQU				$D							;paused in the leadin
STATUS_TRAYMOVING	EQU			$E							;the disc door mechanism is in operation

;drive request format nibbles

RF_A			EQU				$0							;request absolute time
RF_R			EQU				$1							;request relative time
RF_T			EQU				$2							;request track info
RF_TOCO			EQU				$3							;request disc completion time
RF_TOCT			EQU				$4							;request start/end tracks on disc
RF_TOCN			EQU				$5							;request start time of track N
RF_E			EQU				$6							;Error information requested
RF_SUBCERROR	EQU				$E							;had subcode error
RF_NOTREADY		EQU				$F							;not ready to comply with command at the moment

;request Flags field definitions

FLAGS_DATA		EQU				$4
FLAGS_AUDIO		EQU				$0
FLAGS_aEMPHASIS	EQU				$2
FLAGS_iEMPHASIS	EQU				$0
FLAGS_aMUTE		EQU				$1
FLAGS_iMUTE		EQU				$0

;TOCN flag that indicates that track is data or audio

TOCN_FLAG_DATA	EQU				$80							;this flag gets added to high bit of frames info
TOCN_FLAG_AUDIO	EQU				$00

;defines for various values sent to, and received by the drive mechanism (IBM)

;commands to the drive mechanism

MC_RESET		EQU				0							;reset the mechanism
MC_STOP			EQU				1							;tells mechanism to spin down (stop)
MC_PLAY			EQU				2							;play from current paused location, or at end of next seek
MC_PAUSE		EQU				3							;pause play at current location, or end of next seek
MC_SEEK			EQU				4							;seek to specified location and play or pause depending on the current mode
MC_SKIP			EQU				5							;skip forwards, or backwards the given number of "tracks", and play or pause depending on the current mode
MC_OPEN			EQU				6							;open the door
MC_CLOSE		EQU				7							;close the door
MC_COMMANDPACKET	EQU			8							;receive command packet

MAXMBYTES		EQU				10							;maximum number of bytes in communication field to mechanism
SUBQLENGTH		EQU				10							;number of bytes of subQ received from mechanism

;mechanism status bits/fields

MS_ERROR		EQU				$C0							;mask for error bits in status
MS_NOERROR		EQU				$00							;no error to report
MS_FOCUSERROR	EQU				$40							;had focusing error
MS_TRACKINGERROR	EQU			$80							;had tracking error
MS_TOTALERROR	EQU				$C0							;something seriously bad happened

MS_DOOR			EQU				$30							;mask for door status
MS_OPENING		EQU				$00							;door is opening
MS_OPEN			EQU				$10							;door is open
MS_CLOSING		EQU				$20							;door is closing
MS_CLOSED		EQU				$30							;door is closed

MS_FOCUS		EQU				$0C							;focus status of laser playback mechanism
MS_NOTFOCUSING	EQU				$00							;not trying to focus (can only occur when servo is stopped)
MS_FOCUSING		EQU				$04							;focus in progress, but not focussed (can only occur with door closed)
MS_INFOCUS		EQU				$08							;focus has been obtained (can only occur with door closed)

MS_SERVO		EQU				$03							;this field tells the state of the servo
MS_STOP			EQU				$00							;drive is spun down, doing nothing
MS_SEEK			EQU				$01							;seeking/spinning up
MS_PAUSE		EQU				$02							;spun up, and paused
MS_PLAY			EQU				$03							;spun up, and playing

;
;internal states for emulation state machine on 6303
;

ES_STOPPING		EQU				0							;while stopping, we will be in this state
ES_TOCING		EQU				1							;while getting the TOC and program area offset, this will be the state
ES_READING		EQU				2							;during execution of ROM read, this state will be active
ES_SEEKING		EQU				3							;while processing a seek command
ES_PAUSING		EQU				4							;while the drive is paused
ES_PLAYING		EQU				5							;while the drive is playing
ES_FWDING		EQU				6							;while skipping forward
ES_RVSING		EQU				7							;while skipping backard
ES_TSKPING		EQU				8							;skipping across the disc
ES_CUING		EQU				9							;searching for a track
ES_CLOSING		EQU				10							;while closing the door
ES_CLOSED		EQU				11							;while door is closed
ES_OPENING		EQU				12							;opening the door
ES_OPEN			EQU				13							;while door is open

;------------------------------------------------------------------------------------------------------------
;disc related definitions
;------------------------------------------------------------------------------------------------------------

QCONTROLADR		EQU				0							;offsets within Q data of various things
QTNO			EQU				1							;track number
QPOINT			EQU				2							;index or track number if in TOC
QMIN			EQU				3							;minutes of current track
QSEC			EQU				4							;seconds
QBLK			EQU				5							;blocks
QPMIN			EQU				7							;TOC Absolute track start time
QAMIN			EQU				7							;Absolute time when not in TOC
QPSEC			EQU				8
QASEC			EQU				8
QPBLK			EQU				9
QABLK			EQU				9

CONTROLMASK		EQU				$F0							;mask off which bits are control
CONTROL2CHAN	EQU				$00							;two channels of audio
CONTROL4CHAN	EQU				$80							;four channels of audio
CONTROLNOPREEMPH	EQU			$00							;no pre emphasis on audio
CONTROLPREEMPH	EQU				$10							;audio contains preemphasis
CONTROLAUDIO	EQU				$00							;track is audio
CONTROLDATA		EQU				$40							;track is data
CONTROLNOCOPY	EQU				$00							;track is copy protected
CONTROLCOPY		EQU				$20							;track is copyable

ADRMASK			EQU				$0F							;mask off which bits are ADR
ADRMODE1		EQU				$01							;mode 1 subcode data
ADRMODE2		EQU				$02							;mode 2 subcode data
ADRMODE3		EQU				$03							;mode 3 subcode data

LEADINNUM		EQU				0							;the magical track numbers for leadin, and leadout
LEADOUTNUM		EQU				$AA

TIME.min		EQU				0							;offsets to elements of time structure
TIME.sec		EQU				1
TIME.blk		EQU				2
TIME.size		EQU				3

;disc table of contents structure definitions

TOCEntry.controlAdr	EQU			0							;control/adr info from Q subcode
TOCEntry.point	EQU				TOCEntry.controlAdr+1		;the POINT in binary (NOT BCD)
TOCEntry.pMinute	EQU			TOCEntry.point+1			;PMIN,PSEC,PFRAME in binary
TOCEntry.pSecond	EQU			TOCEntry.pMinute+1
TOCEntry.pFrame	EQU				TOCEntry.pSecond+1
TOCEntry.size	EQU				TOCEntry.pFrame+1

TOC.firstTrack	EQU				0
TOC.lastTrack	EQU				TOC.firstTrack+1
TOC.numTracks	EQU				TOC.lastTrack+1
TOC.leadoutStartBlock	EQU		TOC.numTracks+1				;if program blocks start at 0, this is the first non-program block in image
TOC.A0Entry		EQU				TOC.leadoutStartBlock+4		;information about first track
TOC.A1Entry		EQU				TOC.A0Entry+TOCEntry.size	;information about last track
TOC.A2Entry		EQU				TOC.A1Entry+TOCEntry.size	;information about leadout area
TOC.trackEntries	EQU			TOC.A2Entry+TOCEntry.size	;toc entries for maximum number of disc tracks
TOC.size		EQU				TOC.trackEntries+99*TOCEntry.size	;the size of the table of contents


TOC.programAreaOffset	EQU		TOC.leadoutStartBlock+4		;offset to first non-TOC block in disc image

;------------------------------------------------------------------------------------------------------------
;address locations for various things
;------------------------------------------------------------------------------------------------------------

;current memory map
;
;8000-83FF	mechanism interface
;8400-FFFF	stack, non zeropage globals, program
;

;system stack size

STACKSIZE		EQU				1024						;number of bytes to reserve for the stack

MECHANISMBASE	EQU				EXTERNALRAMSTART			;base location for all mechanism registers (use the first bank of RAM)
MAINBASE		EQU				EXTERNALRAMSTART+1024		;start at next 1K boundary
