$CHIP=6301		;select 6301 chip
$NOLIST			;don't want a list at assembly time

				%INCLUDE		"equates.asm"
;
;Zero page
;
				DEFSEG			zeroSeg,START=INTERNALRAMSTART,CLASS=PAGE0
				SEG				zeroSeg

				PUBLIC			ctrl0Shadow
				PUBLIC			ctrl1Shadow
				PUBLIC			ctrl2Shadow
				PUBLIC			sourcePtr
				PUBLIC			destinationPtr
				PUBLIC			LEDTimer
				PUBLIC			bankCounter
				PUBLIC			scanClock
				PUBLIC			lastMechanismStatus
				PUBLIC			driveStatus
				PUBLIC			errorStatusValid
				PUBLIC			errorStatus
				PUBLIC			statusBuffer
				PUBLIC			commandBuffer
				PUBLIC			statusCommandCounter
				PUBLIC			statusSendFlag
				PUBLIC			commandReceiveFlag
				PUBLIC			commandValidFlag
				PUBLIC			TOCNTrackNumber
				PUBLIC			TOCItemNeeded
				PUBLIC			haveTOCValid
				PUBLIC			haveTOC
				PUBLIC			havePAValid
				PUBLIC			havePA
				PUBLIC			PABlock
				PUBLIC			haveDiscValid
				PUBLIC			haveDisc
				PUBLIC			emulationState
				PUBLIC			statusOutputValid
				PUBLIC			requestFormat
				PUBLIC			tempTime
				PUBLIC			tempBlock

ctrl0Shadow:
				DS				1
ctrl1Shadow:
				DS				1
ctrl2Shadow:
				DS				1
sourcePtr:
				DS				2
destinationPtr:
				DS				2
LEDTimer:
				DS				1
bankCounter:
				DS				1							;counts up at every bank select
scanClock:
				DS				1							;counts toward zero, used during scan operations
lastMechanismStatus:
				DS				1							;used in state machine where multiple accesses to stable mechanism status are required
driveStatus:
				DS				1							;current drive status (sans error information)
errorStatusValid:
				DS				1							;tells if we have an overriding error status to report
errorStatus:
				DS				1							;current error status (if applicable)
statusBuffer:
				DS				STATUSBUFFERLENGTH
commandBuffer:
				DS				COMMANDBUFFERLENGTH
statusCommandCounter:
				DS				1							;counts nibbles while sending status, or getting command
statusSendFlag:
				DS				1							;TRUE if status was sent last time
commandReceiveFlag:
				DS				1							;TRUE if a command was received last time
commandValidFlag:
				DS				1							;TRUE if the command that was received had good checksum
TOCNTrackNumber:
				DS				1							;when report request wants start time of a track, the track number is kept here
TOCItemNeeded:
				DS				1							;hold the TOC point that we wish to grab next
haveTOCValid:
				DS				1							;TRUE if the state of the TOC has been determined
haveTOC:
				DS				1							;tells if we have a TOC or not (only meaningful if haveTOCValid=TRUE)
havePAValid:
				DS				1							;TRUE if the state of the first block of program area has been determined
havePA:
				DS				1							;tells if we have the first block or not (only meaningful if havePAValid=TRUE)
PABlock:
				DS				4							;the current block while getting program area offset
haveDiscValid:
				DS				1							;TRUE if the state of the disc in the drive has been determined
haveDisc:
				DS				1							;tells if there is a disc in the drive (only meaningful if haveDiscValid=TRUE)
emulationState:
				DS				1							;the current state of the emulation state machine
statusOutputValid:
				DS				1							;tells if current status, or 0s should be sent (special case at startup to match drive behavior)
requestFormat:
				DS				1							;current request format
tempTime:
				DS				TIME.size					;temporary storage for MM:SS:FF time
tempBlock:
				DS				4							;temporary storage for block number during calculations


;------------------------------------------------------------------------------------------------------------
;
;pseudo hardware locations for communicating with the drive mechanism (IBM)
;
				DEFSEG			mechanismLocation,START=MECHANISMBASE
				SEG				mechanismLocation

				PUBLIC			m_Checksum
				PUBLIC			m_Alive
				PUBLIC			m_ReadyForCommand
				PUBLIC			m_Status
				PUBLIC			m_WroteSubQ
				PUBLIC			m_SubQBuffer
				PUBLIC			m_SubQCRC
				PUBLIC			m_SeekAddress
				PUBLIC			m_SkipDirection
				PUBLIC			m_SkipCount
				PUBLIC			m_ByteCount
				PUBLIC			m_Bytes

m_Checksum		DS				2							;checksum used in restart procedure
m_Alive			DS				1							;TRUE if mechanism is up
m_ReadyForCommand	DS			1							;TRUE if mechanism is ready for a new command
m_Status		DS				1							;the current state of the disc mechanism
m_WroteSubQ		DS				1							;when valid subcode arrives, this will be set
m_SubQBuffer	DS				SUBQLENGTH					;subcode Q buffer
m_SubQCRC		DS				1							;boolean that tells if CRC for the bytes in subCodeQBuffer was ok or not
m_SeekAddress	DS				4							;buffer that contains the absolute disc address to seek to (longword)
m_SkipDirection	DS				1							;when skipping, this indicates the direction: 0 = forward (outward), non-0 = backward (inward)
m_SkipCount		DS				2							;buffer that contains the number of "tracks" to skip
m_ByteCount		DS				1							;number of bytes in command communication buffer
m_Bytes			DS				MAXMBYTES					;communication buffer

;------------------------------------------------------------------------------------------------------------
;
;Other globals
;
				DEFSEG			mainSeg,START=MAINBASE
				SEG				mainSeg

				PUBLIC			stackBase
				PUBLIC			programAreaOffset
				PUBLIC			theTOC

stackTop:
				DS				STACKSIZE-1					;allocate all bytes of the stack except the last one
stackBase:
				DS				1							;set base pointing to the last one
programAreaOffset:
				DS				4							;physical block number of first block of program area on disc image
theTOC:
				DS				TOC.size					;number of bytes needed to contain the TOC structure

				END
