$CHIP=6301		;select 6301 chip
$NOLIST			;dont want list output

				%INCLUDE		"equates.asm"
				%INCLUDE		"macro.asm"
				%INCLUDE		"globals.ext"
				%INCLUDE		"convert.ext"

				DEFSEG			mainSeg
				SEG				mainSeg


GetTOCTrack		PROC
;see if the TOC subcode buffer contains an entry for the passed track
;if it does, extract the information into the TOC, and return TRUE
;if it does not, return FALSE
;INPUT
;	A	the track number we are looking for
;	X	the place to store the entry if we get it
;OUTPUT
;	A	TRUE if it was found, and extracted, FALSE if not

				PSHX
				PSHA										;save the track number for a moment
				LDAA			m_SubQBuffer+QPOINT			;get to the point
				JSR				BCDToBinary					;convert to binary
				TAB											;save result here for a moment
				PULA										;get track number we wanted back again
				PULX
				CBA											;is it the one that we wanted?
				BNE				..fail
				LDAA			m_SubQBuffer+QCONTROLADR	;copy control information
				STAA			TOCEntry.controlAdr,X

				LDAA			m_SubQBuffer+QPOINT			;copy the track number
				PSHX										;save this
				JSR				BCDToBinary					;make into binary
				PULX
				STAA			TOCEntry.point,X

				LDAA			m_SubQBuffer+QPMIN			;copy absolute starting address of track
				PSHX										;save this
				JSR				BCDToBinary					;make into binary
				PULX
				STAA			TOCEntry.pMinute,X

				LDAA			m_SubQBuffer+QPSEC
				PSHX										;save this
				JSR				BCDToBinary					;make into binary
				PULX
				STAA			TOCEntry.pSecond,X

				LDAA			m_SubQBuffer+QPBLK
				PSHX										;save this
				JSR				BCDToBinary					;make into binary
				PULX
				STAA			TOCEntry.pFrame,X

				LDAA			#TRUE
				BRA				..exit
..fail:
				LDAA			#FALSE						;report failure to get the track that was requested
..exit:
				RTS
				ENDPROC


GetNextTOCInput	PROC
;if we are getting the TOC from the disc, and it has been determined that
;there is valid TOC subQ waiting in the subcode Q buffer, see if the subQ contains
;the next item we wish to extract from the TOC
;if it does, extract it, and return TRUE
;if the item is not what we are looking for just ignore it, and return FALSE

				LDAA			TOCItemNeeded				;find out what item is needed
				BEQ				..notDone					;cannot locate this item, so return FALSE
				CMPA			#100						;see if track number
				BCC				..isSpecial					;it is one of the special codes, see which one
;want to get TOC entry for a given track
				DECA										;tracks start at 1, table starts with entry 0
				LDAB			#TOCEntry.size				;get the size of a toc entry
				MUL											;get index to table in D
				ADDD			#theTOC+TOC.trackEntries	;get offset to start of track entry we are interested in
				XGDX										;place it into index register
				LDAA			TOCItemNeeded				;get the item back again
				JSR				GetTOCTrack					;attempt to get this item
				BRA				..exit						;pass back boolean from above
..isSpecial:
				CMPA			#$A0
				BNE				..notA0
				LDX				#theTOC+TOC.A0Entry			;this is the entry to fill in
				JSR				GetTOCTrack					;attempt to get this item
				BRA				..exit						;pass back boolean from above
..notA0:
				CMPA			#$A1
				BNE				..notA1
				LDX				#theTOC+TOC.A1Entry			;this is the entry to fill in
				JSR				GetTOCTrack					;attempt to get this item
				BRA				..exit						;pass back boolean from above
..notA1:
				CMPA			#$A2
				BNE				..notDone					;some sort of invalid request, so return FALSE
				LDX				#theTOC+TOC.A2Entry			;this is the entry to fill in
				JSR				GetTOCTrack					;attempt to get this item
				BRA				..exit						;pass back boolean from above
..notDone:
				LDAA			#FALSE
..exit:
				RTS
				ENDPROC


DoTOCStates		PROC
;if we are getting the TOC from the disc, and it has been determined that
;there is valid TOC subQ waiting in the subcode Q buffer, see if the subQ contains
;the next item we wish to extract from the TOC
;if it is not the item we want, return FALSE
;if it is the item, extract it, and set up for next time, if it was not the
;last item, return FALSE
;if it was the last item to extract, return TRUE

				JSR				GetNextTOCInput				;get the next bit of the TOC if possible
				TSTA										;see if we got it
				BEQ				..notDone
				LDAA			TOCItemNeeded				;find out what item we just got
				CMPA			#100
				BCC				..wasSpecial
				CMPA			theTOC+TOC.lastTrack		;see if we got the last one
				BCC				..done						;it was the last one, so leave
				INCA										;bump up to next track
				STAA			TOCItemNeeded				;this is needed next time
				BRA				..notDone
..wasSpecial:
				CMPA			#$A0
				BNE				..notA0
				LDAA			#$A1
				STAA			TOCItemNeeded				;this is the next item we want
				BRA				..notDone					;pass back boolean from above
..notA0:
				CMPA			#$A1
				BNE				..notA1
				LDAA			#$A2
				STAA			TOCItemNeeded				;this is the next item we want
				BRA				..notDone					;pass back boolean from above
..notA1:
				CMPA			#$A2
				BNE				..notDone					;some sort of invalid request, so return FALSE
				LDAB			theTOC+TOC.A0Entry+TOCEntry.pMinute	;this is the first track on the disc
				STAB			theTOC+TOC.firstTrack
				STAB			TOCItemNeeded				;it is also the next TOC item we wish to get

				LDAA			theTOC+TOC.A1Entry+TOCEntry.pMinute	;this is the last track on the disc
				STAA			theTOC+TOC.lastTrack
				SBA											;determine number of tracks
				INCA
				STAA			theTOC+TOC.numTracks		;store it away
				LDX				#theTOC+TOC.leadoutStartBlock	;destination block number
				PSHX
				LDX				#theTOC+TOC.A2Entry+TOCEntry.pMinute	;source atime
				PSHX
				JSR				ATimeToBlks					;do the conversion
				PULX										;fix the stack after the call
				PULX
				BRA				..notDone					;not done yet, keep going next time
..done:
				LDAA			#TRUE
				BRA				..exit			
..notDone:
				LDAA			#FALSE
..exit:
				RTS
				ENDPROC


				PUBLIC			GetTOC
GetTOC			PROC
;if the TOC is being played at the moment
;this will be called to handle input of the actual TOC
;this will return TRUE when it is complete.
;if TRUE is returned, the global haveTOC will be set to indicate success
;or failure
				LDAA			m_SubQCRC					;is subcode valid?
				BEQ				..notDone					;no, so wait for next block to come by
				LDAA			m_SubQBuffer+QCONTROLADR	;see if TOC info, or something else, (ignore anything else)
				ANDA			#ADRMASK					;mask off the ADR
				CMPA			#ADRMODE1					;mode1?
				BNE				..notDone					;not proper mode, so just ignore it
				LDAA			m_SubQBuffer+QTNO			;had better still be in leadin, or else we fail to get the TOC
				CMPA			#LEADINNUM					;see if in leadin
				BNE				..fail						;fell out of leadin, so we have failed
				JSR				DoTOCStates					;try to get the next item of TOC
				TSTA										;if done, tell the world
				BNE				..succeed
..notDone:
				LDAA			#FALSE						;return FALSE if not done yet
				BRA				..exit
..succeed:
				LDAA			#TRUE						;we are done, set haveTOC to TRUE, and return TRUE
				STAA			haveTOC
				BRA				..exit
..fail
				LDAA			#FALSE						;we are done, but have failed, so set haveTOC FALSE, and return TRUE
				STAA			haveTOC
				LDAA			#TRUE
..exit
				RTS
				ENDPROC


				PUBLIC			InitTOCRead
InitTOCRead		PROC
;before calling GetTOC for the first time, this must be called to set up the
;TOC read state machine
				LDAA			#FALSE
				STAA			haveTOCValid				;do not have TOC yet
				STAA			haveTOC
				LDAA			#$A0
				STAA			TOCItemNeeded				;this is the first TOC item we wish to get
				RTS
				ENDPROC


TestProgramArea	PROC
;while searching for the start of the program area, and the mechanism is
;in PLAY, and valid mode1 subQ is found, call this to see if we have found the
;program area
;at entry, PABlock is the physical block number that corresponds to the subcode
;TRUE is returned if the start of the program area has been located, and
;the location is written into the proper global

				LDAA			m_SubQBuffer+QTNO			;check to see if in leadin still
				CMPA			#LEADINNUM					;see if in leadin
				BEQ				..fail						;was in leadin, so try again next time
;got non-leadin space, so get the absolute time from it, and
;determine what physical block must be the start of the program area

				LDAA			m_SubQBuffer+QAMIN			;get binary version of atime
				JSR				BCDToBinary
				STAA			tempTime+TIME.min			;store the minutes

				LDAA			m_SubQBuffer+QASEC
				JSR				BCDToBinary
				STAA			tempTime+TIME.sec			;store the seconds

				LDAA			m_SubQBuffer+QABLK
				JSR				BCDToBinary
				STAA			tempTime+TIME.blk			;and blocks

				LDX				#tempBlock					;destination
				PSHX
				LDX				#tempTime+TIME.min			;convert time into physical blocks
				PSHX
				JSR				ATimeToBlks					;get physical blocks
				PULX
				PULX

;now subtract the block number from the temp block number to get the offset
				LDD				PABlock+2					;get low portion of absolute block number
				SUBD			tempBlock+2					;subtract out low
				STD				programAreaOffset+2			;store the low portion
				LDD				PABlock						;get high portion
				BCC				..noCarry
				SUBD			#1							;account for the carry
..noCarry:
				SUBD			tempBlock					;subtract high portion of this
				STD				programAreaOffset			;save the high portion

				LDAA			#TRUE						;we are done
				BRA				..exit
..fail
				LDAA			#FALSE						;not in program area yet
..exit
				RTS
				ENDPROC


				PUBLIC			GetProgramArea
GetProgramArea	PROC
;while searching for the start of the program area, and the mechanism is
;in PLAY, call this to see if we have found the
;program area
;TRUE is returned if we want to leave, or skip to the next block
;if FALSE is returned, the subQ meant nothing, and the next block should be read
;when TRUE is returned, the havePA global can be checked to see if the program area was
;located or not, if it was, the location process can stop
;if it was not located, the mechanism should be made to search forward to PABlock
;and when play again commences, this should be called.

				LDAA			m_SubQCRC					;is subcode valid?
				BEQ				..notDone					;no, so wait for next block to come by
				LDAA			m_SubQBuffer+QCONTROLADR	;looking for track number, ignore catalog/isrc
				ANDA			#ADRMASK					;mask off the ADR
				CMPA			#ADRMODE1					;mode1?
				BNE				..notDone					;not proper mode, so just ignore it, go on to next block
				JSR				TestProgramArea				;see if in program area
				TSTA										;if done, tell the world
				BNE				..succeed
				
				LDD				PABlock+2					;increment the current physical block number for next time
				ADDD			#250						;skip forward to this block next time
				STD				PABlock+2
				BCC				..noCarry1
				LDD				PABlock
				ADDD			#1
				STD				PABlock
..noCarry1:
				BRA				..fail						;tell outside we need to skip blocks
..notDone:
				LDD				PABlock+2					;increment the current physical block number for next time
				ADDD			#1
				STD				PABlock+2
				BCC				..noCarry2
				LDD				PABlock
				ADDD			#1
				STD				PABlock
..noCarry2:
				LDAA			#FALSE						;return FALSE if not done yet
				BRA				..exit
..succeed:
				LDAA			#TRUE						;we are done, set havePA to TRUE, and return TRUE
				STAA			havePA
				BRA				..exit
..fail
				LDAA			#FALSE						;we are done, but have failed, so set haveTOC FALSE, and return TRUE
				STAA			havePA
				LDAA			#TRUE
..exit
				RTS
				ENDPROC


				PUBLIC			InitGetProgramArea
InitGetProgramArea	PROC
;before calling GetProgramArea for the first time, this must be called to set up the
;state machine
				LDAA			#FALSE
				STAA			havePAValid					;do not have PA yet
				STAA			havePA
				CLRA
				CLRB
				STD				PABlock						;clear high portion of search block
				STD				PABlock+2					;clear low portion of search block
				RTS
				ENDPROC


				PUBLIC			GetTrackPBlock
GetTrackPBlock	PROC
;once the TOC has been successfully read in, this may be called to get the physical
;block location of any track in the image
;INPUT
;	A		the track number whose physical block location is required
;	X		pointer to the place to store the physical block number
				PSHX										;hold the destination
				PSHX										;pass the destination
				DECA										;tracks start at 1, table starts with entry 0
				LDAB			#TOCEntry.size				;get the size of a toc entry
				MUL											;get index to table in D
				ADDD			#theTOC+TOC.trackEntries+TOCEntry.pMinute	;get offset to start time of the track we are interested in
				XGDX										;place it into index register
				PSHX										;pass this as the source
				JSR				ATimeToBlks					;convert MM:SS:FF to logical block number
				PULX										;remove parameters from stack
				PULX
				PULX										;get destination back again
				JSR				LBlockToPBlock
				RTS
				ENDPROC

				END
