$CHIP=6301		;select 6301 chip
$NOLIST			;dont want list output
;
;Rules for passed parameters that are followed throughout
;this code, unless otherwise documented.
;1. A routine can destroy any register.
;2. Pass pointers in X, byte values in A, word values in D
;3. If the routine requires more input than can be managed easily with the
;	limited set of registers available, pass parameters on the stack 'C' style
;	'C' style means that the parameters are placed on the stack in reverse order,
;	and that the caller is responsible for restoring the stack after the call
;
;Return booleans and bytes in A, word values in D, pointers in X
;

				%INCLUDE		"globals.ext"
				%INCLUDE		"host2ibm.ext"
				%INCLUDE		"toc.ext"
				%INCLUDE		"convert.ext"
				%INCLUDE		"equates.asm"
				%INCLUDE		"macro.asm"

				DEFSEG			mainSeg
				SEG				mainSeg

;------------------------------------------------------------------------------------------------------------
;utility routines
;------------------------------------------------------------------------------------------------------------

				PUBLIC			RamTest
RAMTest			PROC
;Test internal RAM by writing a repeated sequence of numbers into it
;then reading the sequence back to make sure it is valid
;
;If there is a failure, return FALSE, otherwise, return TRUE
				LDX				#INTERNALRAMSTART
StartCnt1:
				CLRA
RAMWrite:
				CPX				#INTERNALRAMEND
				BCC				RAMFull						;numbers have been written
				STAA			0,X
				INX											;point to next location for next time
				INCA
				CMPA			#127						;use a prime number here
				BNE				RAMWrite					;keep going
				BRA				StartCnt1					;re-load the start count
RAMFull:
				LDX				#INTERNALRAMSTART			;point back to start for compare
StartCnt2:
				CLRA
RAMTestBeg:
				CPX				#INTERNALRAMEND
				BCC				RAMPass						;RAM has passed test
				CMPA			0,X							;make sure this matches
				BNE				RAMFail						;if not, go complain
				INX											;point to next location for next time
				INCA
				CMPA			#127						;use a prime number here
				BNE				RAMTestBeg					;keep going
				BRA				StartCnt2					;re-load the start count
RAMPass:
				LDAA			#TRUE
				BRA				RAMTestExit
RAMFail:
				LDAA			#FALSE
RAMTestExit:
				RTS
				ENDPROC


TestHardware	PROC
;Test all the hardware
;If there is a failure at any point, stop, and return FALSE
;otherwise, return TRUE
				JSR				RAMTest
				RTS
				ENDPROC

InitHardwareForTest	PROC
;Set up the hardware for testing
;This must be called with IRQs disabled
;If there was a problem, return FALSE
				CLRA	
				STAA			PORT2DDR					;set all the data direction registers
				STAA			PORT5DDR					;to input first
				STAA			PORT6DDR

				LDAA			#STBYPWR+RAME+STBYFLAG+HALTE	;Set the port 5 control register (allow internal RAM, no slow memory access, allow HALT)
				STAA			RP5CR

				LDAA			#TRUE						;all is well
				RTS
				ENDPROC


InitHardwareForEmulation	PROC
;Set up the hardware for emulation
;This must be called with IRQs disabled
;If there was a problem, return FALSE
				CLRA	
				STAA			PORT2DDR					;set all the data direction registers
				STAA			PORT5DDR					;to input first
				STAA			PORT6DDR
;
;Setup the communications between the IBM and 6303
;
;set up the 3 ports on the 6303
;

				LDAA			#STBYPWR+RAME+STBYFLAG+HALTE	;Set the port 5 control register (allow internal RAM, no slow memory access, allow HALT)
				STAA			RP5CR

				LDAA			#CR2_aBANKIRQ+CR2_iH2ISTROBE+CR2_iH2ISHAKE	;activate 75 Hz. IRQ
				STAA			ctrl2Shadow
				STAA			CTRL_REG2
				LDAA			#CR2_mBANKIRQ+CR2_mH2ISTROBE+CR2_mH2ISHAKE
				STAA			PORT5DDR					;setup port 5 data direction register

				LDAA			#0							;do not generate any strobes while we set things up
				STAA			P6CSR
				CLRA										;set initial data for port 6 to 0
				STAA			PORT6						;write it out
				LDAA			#$FF						;Set port 6 to output
				STAA			PORT6DDR
;
;Ouput Strobe when a write to port 6 and enable output strobe, Input strobe
;interrupt enable on falling edge of P54.
;
				LDAA			#OSS+OSE+ISIRQE
				STAA			P6CSR
;
;Init control register 0, handshake and data lines to the host
;
				LDAA			#CR0_iDATA+CR0_iIRQ+CR0_iCDCK	;set the interface inactive
				STAA			ctrl0Shadow					;save it in its shadow (the shadow begins life here)
				STAA			CTRL_REG0					;and set the hardware register
;
;Init control register 1, Misc. audio control lines and LED!
;
				LDAA			#CR1_iDOUT+CR1_iEMP+CR1_aMUTE+CR1_iC2PO+CR1_iLED	;no data out, no emphasis, mute active, led off
				STAA			ctrl1Shadow					;save it in its shadow
				STAA			CTRL_REG1					;and set the hardware register

				LDAA			RP5CR						;get port 5 control register
				ORAA			#IRQ1E						;enable IRQ1 75 Hz interrupt
				STAA			RP5CR
				LDAA			#TRUE						;return success
				RTS
				ENDPROC


InitGlobals		PROC
;Initialize the globals that will be used throughout emulation
;if there is a problem, return FALSE
				CLRA
				STAA			statusSendFlag				;status not sent yet
				STAA			commandReceiveFlag			;and command not received yet
				STAA			commandValidFlag			;command is not valid yet

				CLRA
				STAA			bankCounter					;start the bank counter off at 0
				STAA			scanClock					;start scan clock at 0

				LDAA			#FALSE						;these are cleared at reset
				STAA			haveTOCValid				;have not tried to get TOC yet
				STAA			haveTOC						;do not have TOC yet
				STAA			havePAValid					;have not tried to get TOC yet
				STAA			havePA						;do not have TOC yet
				STAA			haveDiscValid				;have not tested to see if disc is in the drive yet
				STAA			haveDisc					;do not have disc yet

				LDAA			#0
				STAA			TOCItemNeeded				;not looking for any TOC item at the moment

				LDAA			#STATUS_STOP				;initialize drive status (to 0)
				STAA			driveStatus
				LDAA			#FALSE
				STAA			errorStatusValid			;error status is not valid

				LDAA			#ES_CLOSING					;set emulation state to closing
				STAA			emulationState

				LDAA			#RF_A						;set a format (not used initially)
				STAA			requestFormat

				LDAA			#FALSE						;do not send normal status, send startup status (send all 0s)
				STAA			statusOutputValid

				LDAA			#75
				STAA			LEDTimer

				LDAA			#TRUE						;all is well
				RTS
				ENDPROC


				PUBLIC			StartTimeout
StartTimeout	PROC
;Start a timeout.
;D has the # of processor cycles to count down before timeout is complete.
				SEI
				PSHA										;save part of the timeout value
				LDX				FRCOUNTER
				STX				OCR1						;make sure we can properly clear OCF1
				LDAA			TCSR2						;clear output compare flag 1
				PULA										;recall it
				ADDD			FRCOUNTER					;add to the current value of the free running counter
				STD				OCR1						;set the end value
				CLI
				RTS
				ENDPROC


				PUBLIC			CheckTimeout
CheckTimeout	PROC
;Return TRUE if timeout is complete, else FALSE.
				LDAA			TCSR2						;get timer control/status register
				ANDA			#OCF1						;mask out output compare flag
				RTS											;return the result
				ENDPROC


;------------------------------------------------------------------------------------------------------------
;communications
;------------------------------------------------------------------------------------------------------------

GetSubcodeQ		PROC
;at every frame, there is an implicit command to the mechanism that says
;"give me subcode data for this frame"
;this code waits for the mechanism to give it the subcode
;if the mechanism is not around, this will wait until the bank select IRQ
;goes off, and it will return a subcode buffer, with bad CRC
;NOTE: in order to work properly, this must be called before the next
;frame arrives
				LDAB			bankCounter					;see where we are now so we can tell when new bank IRQ occurs
NoSubQYet:
				LDAA			m_Alive						;see if the mechanism is running
				BEQ				NotAlive
				LDAA			m_WroteSubQ					;it is alive, so see if he wrote some subcode for us
				BNE				GotSubQ						;got subcode for this frame
				TBA
				ADDA			#$2							;see if the mechanism claims to be alive, but did not give us subcode Q
				CMPA			bankCounter
				BNE				NoSubQYet
				BRA				SubQFail					;mechanism failed to give us subcode, so return 0s
NotAlive:
				CMPB			bankCounter					;see if bank is still the same
				BEQ				NotAlive					;if so, just wait here until it changes
SubQFail:
				CLRA										;mechanism is not alive, set subcode as bad, and return it
				STAA			m_SubQCRC					;write the boolean
GotSubQ:
				CLRA										;reset the flag that says new subcode has arrived
				STAA			m_WroteSubQ
				RTS
				ENDPROC

GetMechanismStatus	PROC
;find out what the status of the mechanism is
;if the mechanism is not alive, return the most inactive status
				LDAB			m_Alive						;see if the mechanism is alive
				BEQ				..notAlive
				LDAA			m_Status
				BRA				..exit
..notAlive:
				LDAA			#MS_NOERROR+MS_CLOSED+MS_NOTFOCUSING+MS_STOP	;use this as the status if he is not there
..exit:
				RTS
				ENDPROC


CommandMechanism	PROC
;send a command to the mechanism to tell it what to do next
;seekAddress should have been already set up before this is called, if the
;command that the mechanism is to execute requires an address
;NOTE: if the mechanism reports that it is not alive, this will exit
;the command to execute is passed in A
				LDAB			m_Alive						;see if the mechanism is alive
				BEQ				CantCommandMechanism
				JSR				SendCommandToIBM			;send the command to the mechanism
CantCommandMechanism:
				RTS
				ENDPROC


CheckSum		PROC
;Passed a pointer to the data to checksum in X, checksum the number
;of nibbles given by A, returning the result in A.
				CLRB										;zero starting checksum value
				TSTA										;see if done
				BRA				..LoopEnter					;test for end condition
..LoopTop:
				ADDB			0,X							;sum the next value
				INX											;advance to the next value
				DECA										;count down number left to do
..LoopEnter:
				BNE				..LoopTop					;more to do
				EORB			#$FF						;not to be like drive
				ANDB			#$0F						;only want a nibble
				TBA											;return result in A
				RTS
				ENDPROC


TalkToHost		PROC
;Send the current status to the host, then get a command from the host.
;This will set up the command buffer, and some globals that tell how the communication
;went
				CLRA
				STAA			statusSendFlag				;status not sent yet
				STAA			commandReceiveFlag			;and command not received yet
				STAA			commandValidFlag			;command is not valid yet

				LDX				#statusBuffer				;checksum the status buffer before trying to send it
				LDAA			#STATUSBUFFERLENGTH-1
				JSR				CheckSum
				STAA			statusBuffer+SB_SUM			;put the checksum for the status into the status buffer

				LDAA			STAT_REG0
				ANDA			#SR0_mHSCK					;if HSCK is inactive we are allowed to
				CMPA			#SR0_iHSCK					;communicate, otherwise we are not
				BEQ				..Skip1
				JMP				CommunicationFail			;host is not ready to communicate
..Skip1
				LDD				#IRQTIMEOUT*CYCLESPERMS
				JSR				StartTimeout				;start the IRQ timeout
				SETCR0I			CR0_mIRQ,CR0_aIRQ			;tell the host we are ready to communicate by activating IRQ
WaitComStart:
				LDAA			STAT_REG0
				ANDA			#SR0_mHSCK					;is HSCK active?
				CMPA			#SR0_aHSCK
				BEQ				StartCommunication			;yes
				LDAA			TCSR2
				ANDA			#OCF1
				BEQ				WaitComStart
				JMP				CommunicationFail
StartCommunication:
				SEI
				LDAA			ctrl0Shadow
				ANDA			#NOT(CR0_mIRQ)				;release IRQ
				ORAA			#CR0_iIRQ
				STAA			ctrl0Shadow
				STAA			CTRL_REG0
				CLI

				LDD				#HSCKTIMEOUT*CYCLESPERMS
				JSR				StartTimeout				;start timeout for communications

				LDX				#statusBuffer				;point to the status buffer
				LDAA			#STATUSBUFFERLENGTH			;number of nibbles to send
				STAA			statusCommandCounter
				BRA				PutStatusOut
StatusLoop:
				LDAA			STAT_REG0					;wait here until HSCK is active
				ANDA			#SR0_mHSCK					;or we timeout
				CMPA			#SR0_aHSCK
				BEQ				PutStatusOut
				LDAA			TCSR2						;see if we have timed out
				ANDA			#OCF1
				BEQ				StatusLoop					;no, continue waiting
				JMP				CommunicationFail

PutStatusOut:
				LDAB			0,X							;get a nibble to send
				EORB			#$0F						;nibbles are sent inverted
				SEI
				LDAA			ctrl0Shadow					;send the nibble
				ANDA			#NOT(CR0_mDATA)
				ABA
				STAA			CTRL_REG0

				ANDA			#NOT(CR0_mCDCK)				;activate CDCK
				ORAA			#CR0_aCDCK
				STAA			ctrl0Shadow
				STAA			CTRL_REG0
				CLI
WaitForHSCK:
				LDAA			STAT_REG0					;wait here until HSCK is inactive
				ANDA			#SR0_mHSCK					;or we timeout
				CMPA			#SR0_iHSCK
				BEQ				ContinueStatusLoop
				LDAA			TCSR2						;see if we have timed out
				ANDA			#OCF1
				BEQ				WaitForHSCK					;no, continue waiting
				JMP				CommunicationFail

ContinueStatusLoop:
				SEI
				LDAA			ctrl0Shadow
				ANDA			#NOT(CR0_mCDCK)				;deactivate CDCK
				ORAA			#CR0_iCDCK
				STAA			ctrl0Shadow
				STAA			CTRL_REG0
				CLI
				INX
				DEC				statusCommandCounter		;decr. nibble counter
				BNE				StatusLoop					;more nibbles to send

				SEI
				LDAA			ctrl0Shadow
				ANDA			#NOT(CR0_mDATA)				;deactivate data lines so that they may be driven by the host
				ORAA			#CR0_iDATA
				STAA			ctrl0Shadow
				STAA			CTRL_REG0
				CLI

				LDAA			#TRUE
				STAA			statusSendFlag				;status has been sent
;
;time to receive the command from the host
;
				LDX				#commandBuffer				;point to the command buffer
				LDAB			#COMMANDBUFFERLENGTH		;number of nibbles to receive
CommandLoop:
				LDAA			STAT_REG0					;wait here until HSCK is active
				ANDA			#SR0_mHSCK					;or we timeout
				CMPA			#SR0_aHSCK
				BEQ				GetCommandIn
				LDAA			TCSR2						;see if we have timed out
				ANDA			#OCF1
				BEQ				CommandLoop					;no, continue waiting
				JMP				CommunicationFail
GetCommandIn:
				LDAA			STAT_REG0					;get the data
				ANDA			#SR0_mDATA					;isolate data nibble
				STAA			0,X							;save the command nibble

				SEI
				LDAA			ctrl0Shadow
				ANDA			#NOT(CR0_mCDCK)
				ORAA			#CR0_aCDCK					;set CDCK active
				STAA			ctrl0Shadow
				STAA			CTRL_REG0
				CLI
WaitForHSCK2:
				LDAA			STAT_REG0					;wait here until HSCK is inactive
				ANDA			#SR0_mHSCK					;or we timeout
				CMPA			#SR0_iHSCK
				BEQ				ContinueCommandLoop
				LDAA			TCSR2						;see if we have timed out
				ANDA			#OCF1
				BEQ				WaitForHSCK2				;no, continue waiting
				JMP				CommunicationFail
ContinueCommandLoop:
				SEI
				LDAA			ctrl0Shadow
				ANDA			#NOT(CR0_mCDCK)
				ORAA			#CR0_iCDCK					;set CDCK inactive
				STAA			ctrl0Shadow
				STAA			CTRL_REG0
				CLI
				INX
				DECB										;decr. nibble counter
				BNE				CommandLoop					;more nibbles to be received
;
;We have received the command, see if it was received well
;
				LDAA			#TRUE
				STAA			commandReceiveFlag			;command received (however, it may be invalid)

				LDX				#commandBuffer				;checksum the command buffer to see if it was received well
				LDAA			#COMMANDBUFFERLENGTH-1
				JSR				CheckSum
				CMPA			commandBuffer+CB_SUM		;see if the sums balance
				BNE				CommunicationFail			;no, so we have a command error
				LDAA			#TRUE
				STAA			commandValidFlag			;command that was received was valid

CommunicationFail:
;reset communication interface.
;
				SETCR0I			CR0_mIRQ+CR0_mCDCK+CR0_mDATA,CR0_iIRQ+CR0_iCDCK+CR0_iDATA
				RTS
				ENDPROC


ResetStatusBuffer	PROC
;set the current status to what it should be after a reset
;the drive sends zeroed status at reset
				CLRA										;value to clear with
				LDAB			#STATUSBUFFERLENGTH			;number of bytes to clear
				LDX				#statusBuffer
..A:
				STAA			0,X
				INX
				DECB
				BNE				..A
				RTS
				ENDPROC


SetDriveStatus	PROC
;set the drive status (non error statuses only)
;INPUT
;	A		- new drive status
				STAA			driveStatus
				RTS
				ENDPROC


SetDriveErrorStatus	PROC
;set the drive error status (error statuses only)
;INPUT
;	A		- new drive error status
				STAA			errorStatus
				LDAA			#TRUE
				STAA			errorStatusValid
				RTS
				ENDPROC


SetEmulationState	PROC
;set the new emulation state
;INPUT
;	A		- new emulation state
				STAA			emulationState				;### for now, just blast it in
				RTS
				ENDPROC

DiscValidated	PROC
;test to see if the disc has been validated, if so, return TRUE, if not
;return FALSE
				LDAA			haveDiscValid				;all of these must be TRUE, or we fail
				BEQ				..fail
				LDAA			haveDisc
				BEQ				..fail
				LDAA			haveTOCValid
				BEQ				..fail
				LDAA			haveTOC
				BEQ				..fail
				LDAA			havePAValid
				BEQ				..fail
				LDAA			havePA
				BEQ				..fail
				LDAA			#TRUE
				BRA				..exit						;all were true, we have a valid disc
..fail:
				LDAA			#FALSE
..exit:
				RTS
				ENDPROC


;------------------------------------------------------------------------------------------------------------
;command handling
;------------------------------------------------------------------------------------------------------------


SetControlLines	PROC
;set the mute, data, and emphasis lines based on the subcode, and mechanism status
				JSR				DiscValidated				;see if the disc has been validated yet
				TSTA
				BEQ				..notPlaying				;if not, do not allow audio out
				LDAA			lastMechanismStatus			;what is the mechanism doing?
				ANDA			#MS_SERVO
				CMPA			#MS_PLAY					;if not playing, then always set /DATAOUT,/EMPHASIS,MUTE
				BNE				..notPlaying
				LDAA			m_SubQCRC					;see if the subcode data is valid
				BEQ				..notValid
				LDAA			m_SubQBuffer+QCONTROLADR	;what kind of play is occuring?
				TAB											;hold on to this
				ANDA			#CONTROLDATA				;see if audio or data
				CMPA			#CONTROLDATA
				BNE				..audioPlaying				;if not data, must be audio
				LDAA			#CR1_aDOUT+CR1_iEMP+CR1_aMUTE	;set up for CDROM data
				BRA				..setIt
..audioPlaying:
				TBA											;see if emphasis is active or not
				ANDA			#CONTROLPREEMPH
				CMPA			#CONTROLPREEMPH				;well?
				BNE				..noEmphasis
				LDAA			#CR1_iDOUT+CR1_aEMP+CR1_iMUTE	;emphasized audio
				BRA				..setIt
..noEmphasis:
				LDAA			#CR1_iDOUT+CR1_iEMP+CR1_iMUTE	;non emphasized audio
				BRA				..setIt
..notPlaying:
				LDAA			#CR1_iDOUT+CR1_iEMP+CR1_aMUTE	;if not playing, set /DATAOUT,/EMPHASIS,MUTE
				BRA				..setIt
..notValid:
				SETCR1I			CR1_mMUTE,CR1_aMUTE			;if subcode is bad, mute no matter what we were doing
				BRA				..exit
..setIt:
				SETCR1A			CR1_mDOUT+CR1_mEMP+CR1_mMUTE
..exit:
				RTS
				ENDPROC


Recalibrate		PROC
;seek to the start of the first track on the disc, and go into the current play/pause state
				LDX				#m_seekAddress				;destination
				LDAA			theTOC+TOC.firstTrack		;first track
				JSR				GetTrackPBlock				;get the physical block location for the first track

				LDX				#m_seekAddress				;recalibrate to just before the start of the first track
				JSR				LAdjustPBlockBack

				LDAA			#MC_SEEK					;begin seeking to the location
				JSR				CommandMechanism			;tell mechanism
				RTS
				ENDPROC


;------------------------------------------------------------------------------------------------------------

GetFlags		PROC
;use the current state of mechanism, and subcode to fill in A with
;a Flags field for request commands
				JSR				DiscValidated				;see if the disc has been validated yet
				TSTA
				BEQ				..notPlaying				;if not, do not allow audio out
				LDAA			lastMechanismStatus			;what is the mechanism doing?
				ANDA			#MS_SERVO
				CMPA			#MS_PLAY					;if not playing, then always set FLAGS_AUDIO,FLAGS_iEMPHASIS,FLAGS_aMUTE
				BNE				..notPlaying
				LDAA			m_SubQCRC					;see if the subcode data is valid
				BEQ				..notValid
				LDAA			m_SubQBuffer+QCONTROLADR	;what kind of play is occuring?
				TAB											;hold on to this
				ANDA			#CONTROLDATA				;see if audio or data
				CMPA			#CONTROLDATA
				BNE				..audioPlaying				;if not data, must be audio
				LDAA			#FLAGS_DATA+FLAGS_iEMPHASIS+FLAGS_iMUTE	;set up for CDROM data
				BRA				..exit
..audioPlaying:
				TBA											;see if emphasis is active or not
				ANDA			#CONTROLPREEMPH
				CMPA			#CONTROLPREEMPH				;well?
				BNE				..noEmphasis
				LDAA			#FLAGS_AUDIO+FLAGS_aEMPHASIS+FLAGS_iMUTE	;emphasized audio
				BRA				..exit
..noEmphasis:
				LDAA			#FLAGS_AUDIO+FLAGS_iEMPHASIS+FLAGS_iMUTE	;non-emphasized audio
				BRA				..exit
..notPlaying:
				LDAA			#FLAGS_AUDIO+FLAGS_iEMPHASIS+FLAGS_aMUTE	;if not playing
				BRA				..exit
..notValid:
				LDAA			#FLAGS_AUDIO+FLAGS_iEMPHASIS+FLAGS_aMUTE	;if subcode not valid during play
..exit:
				RTS
				ENDPROC


ClearStatus		PROC
;at the start of communications, the status is cleared
;call this to do that
				CLRA										;set all status bytes to 0
				STAA			statusBuffer+SB_STATUS		;place it into the status buffer
				STAA			statusBuffer+SB_REQCD
				STAA			statusBuffer+SB_MIN
				STAA			statusBuffer+SB_MIN+1
				STAA			statusBuffer+SB_SEC
				STAA			statusBuffer+SB_SEC+1
				STAA			statusBuffer+SB_FRAME
				STAA			statusBuffer+SB_FRAME+1
				STAA			statusBuffer+SB_FLAGS
				RTS
				ENDPROC

RFAbsolute		PROC
;fill in the request area of the statusBuffer with the last valid absolute disc time

				LDAA			m_SubQCRC					;see if the CRC was any good
				BEQ				..notReady					;no, so dont do any copying
				
				LDAA			m_SubQBuffer+QCONTROLADR	;make sure it is mode1
				ANDA			#ADRMASK
				CMPA			#ADRMODE1
				BNE				..notReady					;subcode does not contain information we need here

				LDAA			m_SubQBuffer+QTNO			;make sure it is valid track
				CMPA			#LEADINNUM
				BEQ				..notReady					;not valid during leadin
;### 11/19/91 Drive sends absolute time during leadout, sega would like us to do the same
;### 				CMPA			#LEADOUTNUM
;###				BEQ				..notReady					;not valid during leadout

				LDAA			requestFormat				;get the format type
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer

				LDAA			m_SubQBuffer+QAMIN			;get minutes in BCD
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_MIN			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_MIN+1

				LDAA			m_SubQBuffer+QASEC			;get seconds in BCD
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_SEC			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_SEC+1

				LDAA			m_SubQBuffer+QABLK			;get frames in BCD
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_FRAME		;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_FRAME+1

				JSR				GetFlags
				STAA			statusBuffer+SB_FLAGS
				BRA				..exit
..notReady:
				LDAA			#RF_NOTREADY				;tell host we are not ready
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer
..exit:
				RTS
				ENDPROC


RFRelative		PROC
;fill in the request area of the statusBuffer with the current relative disc time
				LDAA			m_SubQCRC					;see if the CRC was any good
				BEQ				..notReady					;no, so dont do any copying
				
				LDAA			m_SubQBuffer+QCONTROLADR	;make sure it is mode1
				ANDA			#ADRMASK
				CMPA			#ADRMODE1
				BNE				..notReady					;subcode does not contain information we need here

				LDAA			m_SubQBuffer+QTNO			;make sure it is valid track
				CMPA			#LEADINNUM
				BEQ				..notReady					;not valid during leadin
;###				CMPA			#LEADOUTNUM
;###				BEQ				..notReady					;not valid during leadout

				LDAA			requestFormat				;get the format type
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer

				LDAA			m_SubQBuffer+QMIN			;get minutes in BCD
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_MIN			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_MIN+1

				LDAA			m_SubQBuffer+QSEC			;get seconds in BCD
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_SEC			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_SEC+1

				LDAA			m_SubQBuffer+QBLK			;get frames in BCD
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_FRAME		;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_FRAME+1

				JSR				GetFlags
				STAA			statusBuffer+SB_FLAGS
				BRA				..exit
..notReady:
				LDAA			#RF_NOTREADY				;tell host we are not ready
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer
..exit:
				RTS
				ENDPROC


RFTrack			PROC
;fill in the request area of the statusBuffer with the current track information
				LDAA			m_SubQCRC					;see if the CRC was any good
				BEQ				..notReady					;no, so dont do any copying
				
				LDAA			m_SubQBuffer+QCONTROLADR	;make sure it is mode1
				ANDA			#ADRMASK
				CMPA			#ADRMODE1
				BNE				..notReady					;subcode does not contain information we need here

				LDAA			m_SubQBuffer+QTNO			;make sure it is valid track
				CMPA			#LEADINNUM
				BEQ				..notReady					;not valid during leadin
;###				CMPA			#LEADOUTNUM
;###				BEQ				..notReady					;not valid during leadout

				LDAA			requestFormat				;get the format type
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer

				LDAA			m_SubQBuffer+QTNO			;get track number in BCD
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_TNO			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_TNO+1

				LDAA			m_SubQBuffer+QCONTROLADR	;get control/adr
				LSRA										;isolate control portion
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_CTRL		;place it into the status buffer

				JSR				GetFlags
				STAA			statusBuffer+SB_FLAGS
				BRA				..exit
..notReady:
				LDAA			#RF_NOTREADY				;tell host we are not ready
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer
..exit:
				RTS
				ENDPROC


RFTOCO			PROC
;get completion time (leadout start time)
				JSR				DiscValidated				;see if the disc has been validated yet
				TSTA
				BEQ				..notValid					;if not, then report not ready

				LDAA			requestFormat				;get the format type
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer

				LDAA			theTOC+TOC.A2Entry+TOCEntry.pMinute	;get minutes location of leadout in binary
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_MIN			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_MIN+1

				LDAA			theTOC+TOC.A2Entry+TOCEntry.pSecond	;get seconds location of leadout in binary
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_SEC			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_SEC+1

				LDAA			theTOC+TOC.A2Entry+TOCEntry.pFrame	;get frames location of leadout in binary
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_FRAME		;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_FRAME+1

				BRA				..exit
..notValid:
				LDAA			#RF_NOTREADY				;tell host we are not ready yet
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer
..exit
				JSR				GetFlags
				STAA			statusBuffer+SB_FLAGS
				RTS
				ENDPROC


RFTOCT			PROC
;get start and completion track numbers
;this code is interesting, because if the drive is stopped, it will make
;the drive spinup, but if the drive has been spun up, and the toc read, it will not
;stop the current operation
				JSR				DiscValidated				;see if the disc has been validated yet
				TSTA
				BEQ				..notValid					;if not, then report not ready

				LDAA			requestFormat				;get the format type
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer

				LDAA			theTOC+TOC.firstTrack		;get first track in binary
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_FIRSTTNO	;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_FIRSTTNO+1

				LDAA			theTOC+TOC.lastTrack		;get last track on disc
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_LASTTNO		;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_LASTTNO+1

				LDAA			#TOCTVERSION				;get version number
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_VERSION		;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_VERSION+1

				BRA				..exit
..notValid:
				LDAA			#RF_NOTREADY				;tell host we are not ready yet
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer
..exit
				JSR				GetFlags
				STAA			statusBuffer+SB_FLAGS
				RTS
				ENDPROC


RFTOCN			PROC
;want starting time for track N, go into TOC, and get it
				JSR				DiscValidated				;see if the disc has been validated yet
				TSTA
				BEQ				..notValid					;if not, then report not ready

				LDAA			requestFormat				;get the format type
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer

				LDAA			TOCNTrackNumber				;get the track number that is needed
				CMPA			theTOC+TOC.firstTrack		;see if lower than first track
				BCS				..notValid
				CMPA			theTOC+TOC.lastTrack		;see if higher than last track
				BHI				..notValid
				DECA										;tracks start at 1, table starts with entry 0
				LDAB			#TOCEntry.size				;get the size of a toc entry
				MUL											;get index to table in D
				ADDD			#theTOC+TOC.trackEntries	;get offset to start of track entry we are interested in
				XGDX										;place it into index register

				LDAA			TOCEntry.pMinute,X			;get minute location of this track in binary
				PSHX
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				PULX
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_MIN	;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_MIN+1

				LDAA			TOCEntry.pSecond,X			;get seconds location of this track in binary
				PSHX
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				PULX
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_SEC			;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_SEC+1

				LDAA			TOCEntry.pFrame,X			;get frame location of this track in binary
				PSHX
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				PULX
				LDAB			TOCEntry.controlAdr,X		;see if music, or data, and fudge bit into top of frame info (fax of 8/20/91)
				ANDB			#CONTROLDATA				;see if audio or data
				CMPB			#CONTROLDATA
				BNE				..isAudio					;set flag accordingly
				ORAA			#TOCN_FLAG_DATA
				BRA				..addedFlag
..isAudio:
				ORAA			#TOCN_FLAG_AUDIO
..addedFlag:
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_FRAME		;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_FRAME+1

				LDAA			TOCEntry.point,X			;get the track number
				JSR				BinaryToBCD					;convert to BCD for writing into status area
				ANDA			#$0F						;get low nibble of track number
				STAA			statusBuffer+SB_N			;store it in
				BRA				..exit
..notValid:
				LDAA			#RF_NOTREADY				;tell host we are not ready yet
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer
				JSR				GetFlags					;in this case, only get flags if it is an error
				STAA			statusBuffer+SB_FLAGS
..exit
				RTS
				ENDPROC


RFError			PROC
				LDAA			requestFormat				;get the format type
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer

				LDAA			#0							;### for now, report no error
				TAB
				LSRA
				LSRA
				LSRA
				LSRA
				STAA			statusBuffer+SB_ERRORNUMBER	;place it into the status buffer
				ANDB			#$0F
				STAB			statusBuffer+SB_ERRORNUMBER+1

				JSR				GetFlags					;in this case, only get flags if it is an error
				STAA			statusBuffer+SB_FLAGS
..exit
				RTS
				ENDPROC


RFInvalidType	PROC
				LDAA			requestFormat				;get the format type (invalid formats just go out)
				STAA			statusBuffer+SB_REQCD		;place it into the status buffer
				JSR				GetFlags					;in this case, only get flags if it is an error
				STAA			statusBuffer+SB_FLAGS
				RTS
				ENDPROC

FillRequestedStatus PROC
;using the drive status, and the current request mode, set up the status buffer
				LDAA			statusOutputValid			;see if status, or 0s should go out (at start, 0s go out from drive)
				BNE				..sendStatus
				JSR				ClearStatus					;clear the status
				BRA				..exit
..sendStatus:
				LDX				#RFVectorTable				;point to the request format vector table
				LDAB			requestFormat				;find out what type of request the host would like
				ASLB										;make into table index
				ABX											;index into the table
				LDX				0,X							;get the vector
				JSR				0,X							;go to the routine
				LDAA			driveStatus					;get non-error status
				LDAB			errorStatusValid			;see if there is an error status to report (it will supercede normal status)
				BEQ				..noErrorStatus
				LDAA			#RF_NOTREADY				;during errors, request format is cleared
				STAA			statusBuffer+SB_REQCD		;shove it in (overriding normal request format)
				LDAA			errorStatus					;get the error status
..noErrorStatus:	
				STAA			statusBuffer+SB_STATUS		;shove it in (overriding normal status)
				LDAA			#FALSE
				STAA			errorStatusValid			;error status is no longer valid
..exit:
				RTS
RFVectorTable:
;### NOTE: the order of this table must change if the order
;of the request format nibbles changes
				DW				RFAbsolute					;0
				DW				RFRelative					;1
				DW				RFTrack						;2
				DW				RFTOCO						;3
				DW				RFTOCT						;4
				DW				RFTOCN						;5
				DW				RFError						;6
				DW				RFInvalidType
				DW				RFInvalidType
				DW				RFInvalidType
				DW				RFInvalidType
				DW				RFInvalidType
				DW				RFInvalidType
				DW				RFInvalidType
				DW				RFInvalidType
				DW				RFInvalidType
				ENDPROC

;------------------------------------------------------------------------------------------------------------

Stopping		PROC
;during the execution of the stop function, this is called
;repeatedly
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_SERVO					;see if the servo is stopped
				CMPA			#MS_STOP					;if it is not stopped, then do not change the drive status
				BNE				..exit
				LDAA			haveDiscValid				;see if we know whether or not we have a disc
				BEQ				..statusStop				;if we do not know what state the disc is in, then just say we are stopped
				LDAA			haveDisc					;see if we have a disc
				BNE				..statusStop				;have disc, so status is stop
				LDAA			#STATUS_NODISC				;no disc
				JSR				SetDriveStatus				;set the drive status
				BRA				..gotStatus
..statusStop:
				LDAA			#STATUS_STOP				;set status to stop
				JSR				SetDriveStatus				;set the drive status
..gotStatus:
..exit:
				RTS
				ENDPROC


TOCing			PROC
;while reading in the TOC, this will be called repeatedly
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_SERVO					;see if servo has stopped, and stopped trying to focus
				CMPA			#MS_STOP					;if it is stopped, then do not change the drive status
				BNE				..notStopped				;just leave, see if stopped next time
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_FOCUS					;see if not focusing (this means that there is no disc)
				CMPA			#MS_NOTFOCUSING
				BNE				..exit						;still determining if there is a disc, so go away until next time
				LDAA			#FALSE						;set into no disc state
				STAA			haveDisc
				LDAA			#TRUE
				STAA			haveDiscValid
				LDAA			#STATUS_NODISC				;no disc
				JSR				SetDriveStatus				;set the drive status
				BRA				..exit						;tell the world that there is no disc
..notStopped:
				LDAA			haveDiscValid				;see if we know the state already
				BNE				..haveDiscState				;yes, do not set it
				LDAA			#TRUE
				STAA			haveDisc
				STAA			haveDiscValid				;on TOC read, we determine that we have a disc

..haveDiscState:
				LDAA			haveDisc					;see if we actually have a disc (should, unless something bad happens)
				BEQ				..exit						;if not, get out of here
				LDAA			#STATUS_TOCREAD				;if we are not stopped, then we are reading the TOC
				JSR				SetDriveStatus				;set the drive status
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_SERVO					;see if seeking, playing, or paused
				CMPA			#MS_PLAY
				BNE				..exit						;if not playing, then leave
;getting the TOC, and playing at this moment
				LDAA			haveTOCValid				;do we have a TOC yet?
				BNE				..haveTOC
				JSR				GetTOC						;try to read next portion of TOC
				TSTA										;see if done trying to get TOC now
				BEQ				..exit						;not done trying to get TOC, so just leave
				LDAA			#TRUE						;we now know the state of the toc
				STAA			haveTOCValid
				LDD				PABlock
				STD				m_SeekAddress				;seek to the start of disc for next test
				LDD				PABlock+2
				STD				m_SeekAddress+2
				LDAA			#MC_SEEK
				JSR				CommandMechanism			;seek to the start of the test, should already be playing
				BRA				..exit
..haveTOC:
				LDAA			havePAValid					;do we know where the program area is supposed to begin?
				BNE				..gotoPause					;### yikes, this should not happen
				JSR				GetProgramArea				;do next test while trying to locate start of program area
				TSTA										;see what should be done
				BEQ				..exit						;nothing, he wants to keep looking
				LDAA			havePA						;did we get the program area?
				BNE				..gotPA						;yes, set proper global
				LDD				PABlock
				STD				m_SeekAddress				;seek to the start of disc for next test
				LDD				PABlock+2
				STD				m_SeekAddress+2
				LDAA			#MC_SEEK
				JSR				CommandMechanism			;seek to the start of the test, should already be playing
				BRA				..exit
..gotPA:
				LDAA			#TRUE
				STAA			havePAValid					;got it, so tell the world
				JSR				Recalibrate					;seek to the start of the disc, and pause there
..gotoPause:
				LDAA			#MC_PAUSE
				JSR				CommandMechanism			;once the PA area has been located, go into pause
..exit:
				RTS
				ENDPROC


Reading			PROC
Seeking:
Pausing:
Cuing:
Playing:
;while reading, seeking, pausing, cuing, or playing, this will be called repeatedly
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_SERVO					;isolate stop,seek,pause,play
				CMPA			#MS_STOP					;if it is stopped, then do not change the drive status
				BEQ				..exit
				CMPA			#MS_SEEK
				BNE				..notSeeking				;if not seeking, then must be in play, or pause

				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_ERROR					;isolate mechanism error while seeking
				CMPA			#MS_NOERROR	
				BEQ				..noError					;if no error, then assume all is going well, and report tracking in progress
				
				LDAA			#MC_STOP					;otherwise, ran off the end of the disc, so stop, and report the error
				JSR				CommandMechanism
				LDAA			#ES_STOPPING
				JSR				SetEmulationState			;set the emulation state to stoping (skipping is complete)
				BRA				..exit
..noError:

				LDAA			#STATUS_SEEK				;set drive status to seek
				JSR				SetDriveStatus
				BRA				..exit						;done
..notSeeking:
				CMPA			#MS_PAUSE
				BNE				..notPausing				;if not pausing, then must be in play

				LDAA			driveStatus					;if we were paused in the leadout, then stay that way
				CMPA			#STATUS_DISCEND				;well?
				BEQ				..exit						;if so, stay in this state

				LDAA			#STATUS_PAUSE				;set drive status to paused
				JSR				SetDriveStatus
				BRA				..exit						;done
..notPausing:
				LDAA			m_SubQCRC					;see if the subcode data is valid
				BEQ				..inPlay					;if not, just set status to play
				LDAA			m_SubQBuffer+QCONTROLADR	;make sure it is mode1
				ANDA			#ADRMASK
				CMPA			#ADRMODE1
				BNE				..inPlay					;subcode does not contain information we need here
				LDAA			m_SubQBuffer+QTNO			;check to see if in leadout area
				CMPA			#LEADOUTNUM					;leadout?
				BNE				..inPlay
				LDAA			#MC_PAUSE					;if so, pause, and go into disc-end status
				JSR				CommandMechanism
				LDAA			#STATUS_DISCEND
				JSR				SetDriveStatus
				BRA				..exit				
..inPlay
				LDAA			#STATUS_PLAY				;set drive status to playing
				JSR				SetDriveStatus
..exit:
				RTS
				ENDPROC


FWDing			PROC
;while fwding, this will be called repeatedly
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_SERVO					;isolate stop,seek,pause,play
				CMPA			#MS_STOP					;if it is stopped, then do not change the drive status
				BNE				..notStopping
				JMP				..exit
..notStopping:
				CMPA			#MS_SEEK
				BNE				..notSeeking				;if not seeking, then must be in play, or pause

				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_ERROR					;isolate mechanism error while seeking
				CMPA			#MS_NOERROR	
				BEQ				..noError					;if no error, then assume all is going well, and report scanning in progress
				
				LDAA			#MC_STOP					;otherwise, ran off the end of the disc, so stop, and report the error
				JSR				CommandMechanism
				LDAA			#ES_STOPPING
				JSR				SetEmulationState			;set the emulation state to stoping (skipping is complete)
				JMP				..exit
..noError:

				LDAA			#STATUS_SCAN				;set drive status to scanning
				JSR				SetDriveStatus
				JMP				..exit						;done
..notSeeking:
				CMPA			#MS_PAUSE
				BNE				..notPausing				;if not pausing, then must be in play
				LDAA			#STATUS_DISCEND				;the only reason for this to pause is running into leadout
				JSR				SetDriveStatus
				BRA				..exit						;done
..notPausing:
				LDAA			#STATUS_SCAN				;set drive status to scanning
				JSR				SetDriveStatus

				LDAA			m_SubQCRC					;is subcode valid?
				BEQ				..exit						;no, just keep doing what we were doing
				LDAA			m_SubQBuffer+QCONTROLADR	;looking for track number, ignore catalog/isrc
				ANDA			#ADRMASK					;mask off the ADR
				CMPA			#ADRMODE1					;mode1?
				BNE				..exit						;not proper mode, so just ignore it, go on to next block

				LDAA			m_SubQBuffer+QTNO			;check to see if in leadout area
				CMPA			#LEADOUTNUM					;leadout?
				BNE				..notAtEnd
				LDAA			#MC_PAUSE					;if so, pause, and go into disc-end status
				JSR				CommandMechanism
				LDAA			#STATUS_DISCEND				;set into disc-end status
				JSR				SetDriveStatus
				BRA				..exit
..notAtEnd:
				LDAA			scanClock					;see if it is time to seek again
				BNE				..exit						;no, just keep doing what we were doing

				LDAA			m_SubQBuffer+QAMIN			;get binary version of atime
				JSR				BCDToBinary
				STAA			tempTime+TIME.min			;store the minutes

				LDAA			m_SubQBuffer+QASEC
				JSR				BCDToBinary
				STAA			tempTime+TIME.sec			;store the seconds

				LDAA			m_SubQBuffer+QABLK
				JSR				BCDToBinary
				STAA			tempTime+TIME.blk			;and blocks

				LDX				#tempBlock					;destination
				PSHX
				LDX				#tempTime+TIME.min			;convert time into physical blocks
				PSHX
				JSR				ATimeToBlks					;get physical blocks
				PULX
				PULX
				LDX				#tempBlock					;destination
				JSR				LBlockToPBlock				;convert to physical block number

				LDD				tempBlock+2					;get block address low
				ADDD			#FWDOFFSETLOW				;add low portion of the offset
				STD				m_SeekAddress+2				;save as new seek address
				LDD				tempBlock					;get block address high
				BCC				..noCarry
				ADDD			#1
..noCarry:
				ADDD			#FWDOFFSETHIGH				;add in high word
				STD				m_SeekAddress				;set high word of seek address
				LDAA			#MC_SEEK					;begin seeking to the new location, stay in play
				JSR				CommandMechanism			;tell mechanism
				LDAA			#SCANPLAYTIME
				STAA			scanClock					;reset the scan clock
..exit:
				RTS
				ENDPROC


RVSing			PROC
;while rvsing, this will be called repeatedly
;if we run into the leadin, begin play
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_SERVO					;isolate stop,seek,pause,play
				CMPA			#MS_STOP					;if it is stopped, then do not change the drive status
				BNE				..notStopping
				JMP				..exit
..notStopping:
				CMPA			#MS_SEEK
				BNE				..notSeeking				;if not seeking, then must be in play, or pause

				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_ERROR					;isolate mechanism error while seeking
				CMPA			#MS_NOERROR	
				BEQ				..noError					;if no error, then assume all is going well, and report scanning in progress
				
				LDAA			#MC_STOP					;otherwise, ran off the end of the disc, so stop, and report the error
				JSR				CommandMechanism
				LDAA			#ES_STOPPING
				JSR				SetEmulationState			;set the emulation state to stoping (skipping is complete)
				JMP				..exit
..noError:

				LDAA			#STATUS_SCAN				;set drive status to scanning
				JSR				SetDriveStatus
				JMP				..exit						;done
..notSeeking:
				CMPA			#MS_PAUSE
				BNE				..notPausing				;if not pausing, then must be in play
				BRA				..exit						;done
..notPausing:
				LDAA			#STATUS_SCAN				;set drive status to scanning
				JSR				SetDriveStatus

				LDAA			m_SubQCRC					;is subcode valid?
				BEQ				..exit						;no, just keep doing what we were doing
				LDAA			m_SubQBuffer+QCONTROLADR	;looking for track number, ignore catalog/isrc
				ANDA			#ADRMASK					;mask off the ADR
				CMPA			#ADRMODE1					;mode1?
				BNE				..exit						;not proper mode, so just ignore it, go on to next block

				LDAA			m_SubQBuffer+QTNO			;check to see if in leadout area
				CMPA			#LEADINNUM					;leadin?
				BNE				..notAtStart

				JSR				Recalibrate					;seek to the start of the first track, and go into play
				LDAA			#MC_PLAY					;play when you get there
				JSR				CommandMechanism
				LDAA			#ES_PLAYING
				JSR				SetEmulationState			;set the emulation state to playing
				BRA				..exit

..notAtStart:
				LDAA			scanClock					;see if it is time to seek again
				BNE				..exit						;no, just keep doing what we were doing

				LDAA			m_SubQBuffer+QAMIN			;get binary version of atime
				JSR				BCDToBinary
				STAA			tempTime+TIME.min			;store the minutes

				LDAA			m_SubQBuffer+QASEC
				JSR				BCDToBinary
				STAA			tempTime+TIME.sec			;store the seconds

				LDAA			m_SubQBuffer+QABLK
				JSR				BCDToBinary
				STAA			tempTime+TIME.blk			;and blocks

				LDX				#tempBlock					;destination
				PSHX
				LDX				#tempTime+TIME.min			;convert time into physical blocks
				PSHX
				JSR				ATimeToBlks					;get physical blocks
				PULX
				PULX
				LDX				#tempBlock					;destination
				JSR				LBlockToPBlock				;convert to physical block number

				LDD				tempBlock+2					;get block address low
				SUBD			#RVSOFFSETLOW				;subtract low portion of the offset
				STD				m_SeekAddress+2				;save as new seek address
				LDD				tempBlock					;get block address high
				BCC				..noCarry
				SUBD			#1
..noCarry:
				SUBD			#RVSOFFSETHIGH				;subtract off high word
				STD				m_SeekAddress				;set high word of seek address
				LDAA			#MC_SEEK					;begin seeking to the new location, stay in play
				JSR				CommandMechanism			;tell mechanism
				LDAA			#SCANPLAYTIME
				STAA			scanClock					;reset the scan clock
..exit:
				RTS
				ENDPROC


Skipping		PROC
;while skipping tracks, this will be called repeatedly

				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_SERVO					;isolate stop,seek,pause,play
				CMPA			#MS_STOP					;if it is stopped, then do not change the drive status
				BEQ				..exit
				CMPA			#MS_SEEK
				BNE				..notSeeking				;if not seeking, then must be in play, or pause

				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_ERROR					;isolate mechanism error while seeking
				CMPA			#MS_NOERROR	
				BEQ				..noError					;if no error, then assume all is going well, and report tracking in progress
				
				LDAA			#MC_STOP					;otherwise, ran off the end of the disc, so stop, and report the error
				JSR				CommandMechanism
				LDAA			#ES_STOPPING
				JSR				SetEmulationState			;set the emulation state to stoping (skipping is complete)
				BRA				..exit
..noError:
				LDAA			#STATUS_TRACKING			;set drive status to tracking
				JSR				SetDriveStatus
				BRA				..exit						;done
..notSeeking:
				CMPA			#MS_PAUSE
				BNE				..notPausing				;if not pausing, then must be in play (THIS SHOULD NOT HAPPEN!!)

				LDAA			#STATUS_PAUSE				;set drive status to paused
				JSR				SetDriveStatus
				BRA				..exit						;done
..notPausing:
				LDAA			#MC_PAUSE					;if we were playing when we were not supposed to be, PAUSE now!
				JSR				CommandMechanism
				BRA				..exit				
..exit:
				RTS
				ENDPROC


DoorClosing		PROC
;while the door is closing, this will be called repeatedly
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_DOOR					;isolate door state
				CMPA			#MS_CLOSING					;see if it is closing
				BNE				..notClosing
				LDAA			#STATUS_TRAYMOVING			;tell host that the tray is in motion
				JSR				SetDriveStatus
				BRA				..exit						;that is all for now
..notClosing:
				CMPA			#MS_CLOSED
				BNE				..notClosed					;if it is not closing, and it is not closed, do not set the status
				LDAA			#STATUS_STOP				;set status to stop
				JSR				SetDriveStatus				;set the drive status
				LDAA			#RF_A
				STAA			requestFormat				;implicit stop command forces format to absolute
				LDAA			#ES_CLOSED					;set into new emulation state
				JSR				SetEmulationState			;do it
				BRA				..exit
..notClosed:
..exit:
				RTS
				ENDPROC

DoorClosed		PROC
;once the door has completed closing, wait here until told to do something else
				RTS											;nothing to do, but sit here
				ENDPROC

DoorOpening		PROC
;while the door is opening, this will be called repeatedly
;note: if the door begins to close, or appears closed, go into the closed mode
;this is done to emulate the user pushing the door closed on the drive
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_DOOR					;isolate door state
				CMPA			#MS_OPENING					;see if it is opening
				BNE				..notOpening
				LDAA			#STATUS_TRAYMOVING			;tell host that the tray is in motion
				JSR				SetDriveStatus
				BRA				..exit						;that is all for now
..notOpening:
				CMPA			#MS_OPEN
				BNE				..notOpen					;if it is not opening, and it is not open, do not set the status
				LDAA			#STATUS_DOOROPEN			;tell host that the door is open
				JSR				SetDriveStatus
				LDAA			#ES_OPEN					;set into new emulation state
				JSR				SetEmulationState			;do it
				BRA				..exit
..notOpen:
;if not open, and not opening, then it is closed, or closing, and we should not be in this state
;any longer
				LDAA			#ES_CLOSING
				JSR				SetEmulationState			;set the new emulation state
..exit:
				RTS
				ENDPROC

DoorOpen		PROC
;while the door is open, wait here until told to do something else
;NOTE: if the door starts closing or appears closed, change states
				LDAA			lastMechanismStatus			;get recent status of mechanism
				ANDA			#MS_DOOR					;isolate door state
				CMPA			#MS_OPEN					;see if it is open like it should be
				BEQ				..exit						;it is, so just leave
				CMPA			#MS_OPENING
				BEQ				..exit						;this should not occur, but if it does, handle it
				LDAA			#ES_CLOSING					;set into new emulation state
				JSR				SetEmulationState			;do it
..exit:
				RTS
				ENDPROC


ProcessCurrentState	PROC
;given the current state of emulation, the status of the mechanism,
;and the Q subcode, determine the new state of emulation, and the new drive status
;and control bits
				JSR				GetMechanismStatus			;grab the status for this round
				STAA			lastMechanismStatus			;hold onto it for easy reference
				JSR				SetControlLines				;set the mute, data, and emphasis lines based on the subcode, and mechanism status
				LDX				#StateVectorTable			;point to the state vector table
				LDAB			emulationState				;get the state of the drive
				ASLB										;make into table index
				ABX											;index into the table
				LDX				0,X							;get the vector
				JSR				0,X							;go to the routine
StateProcessed:
				JSR				FillRequestedStatus
				RTS											;current state has been processed
StateVectorTable:
;### NOTE: the order of this table must change if the order
;of the emulation states changes

				DW				Stopping
				DW				TOCing
				DW				Reading
				DW				Seeking
				DW				Pausing
				DW				Playing
				DW				FWDing
				DW				RVSing
				DW				Skipping
				DW				Cuing
				DW				DoorClosing
				DW				DoorClosed
				DW				DoorOpening
				DW				DoorOpen

				ENDPROC


;------------------------------------------------------------------------------------------------------------


PassCommandToIBM	PROC
;when a command other than NOP arrives, pass the command buffer to the IBM
				LDAA			commandBuffer+CB_COMMAND0	;get command
				CMPA			#CMD_NOP					;do not send NOP commands to the IBM (might slow him down)
				BEQ				..exit
				LDX				#commandBuffer
				STX				sourcePtr
				LDX				#m_Bytes
				STX				destinationPtr
				LDAB			#COMMANDBUFFERLENGTH
				STAB			m_ByteCount
				BRA				..loopEnter
..copyLoop:
				LDX				sourcePtr
				LDAA			0,X
				INX
				STX				sourcePtr
				LDX				destinationPtr
				STAA			0,X
				INX
				STX				destinationPtr
				DECB
..loopEnter:
				BNE				..copyLoop
				LDAA			#MC_COMMANDPACKET
				JSR				CommandMechanism
..exit:
				RTS
				ENDPROC


CommandTimeToBlock	PROC
;convert the minute second frame information in the command buffer into a
;block number
;INPUT
;	X	place to store the block number
				PSHX										;pass the destination

				LDAA			commandBuffer+CB_MIN		;get BCD address from command buffer
				ASLA										;get high nibble into place
				ASLA
				ASLA
				ASLA
				ORAA			commandBuffer+CB_MIN+1		;tack on low nibble
				JSR				BCDToBinary					;make into binary
				STAA			tempTime+TIME.min			;store the minutes

				LDAA			commandBuffer+CB_SEC		;get BCD address from command buffer
				ASLA										;get high nibble into place
				ASLA
				ASLA
				ASLA
				ORAA			commandBuffer+CB_SEC+1		;tack on low nibble
				JSR				BCDToBinary
				STAA			tempTime+TIME.sec			;store the seconds

				LDAA			commandBuffer+CB_FRAME		;get BCD address from command buffer
				ASLA										;get high nibble into place
				ASLA
				ASLA
				ASLA
				ORAA			commandBuffer+CB_FRAME+1	;tack on low nibble
				JSR				BCDToBinary
				STAA			tempTime+TIME.blk			;and blocks

				LDX				#tempTime+TIME.min			;convert time into physical blocks
				PSHX
				JSR				ATimeToBlks					;get logical blocks
				PULX
				PULX
				RTS
				ENDPROC

ForceRequestFormat	PROC
;Certain request formats get forced to RF_A
;at the start of certain commands
;this code checks the old format, and forces it to the new one if needed
				LDAA			requestFormat				;if request formats that cause action are valid, kill them, and set to RF_A
				CMPA			#RF_TOCT
				BEQ				..setNewFormat
				CMPA			#RF_TOCN
				BNE				..noNewFormat
..setNewFormat:
				LDAA			#RF_A
				STAA			requestFormat
..noNewFormat:
				RTS
				ENDPROC


DoInvalidSum	PROC
;a command has arrived with a bad checksum, set the status appropriately
				LDAA			#STATUS_SUMERROR
				JSR				SetDriveErrorStatus
				RTS
				ENDPROC


DoInvalidCommand	PROC
;an invalid command has arrived, set the status appropriately
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
				RTS
				ENDPROC


DoReset			PROC
;when a reset arrives, this is the first command to get executed
				LDAA			#MC_RESET					;tell the mechanism to reset
				JSR				CommandMechanism
				LDAA			#ES_CLOSING					;set emulation state to closing
				JSR				SetEmulationState
				RTS
				ENDPROC


DoNopCommand	PROC
;a nop command has arrived, so do nothing
				RTS
				ENDPROC


DoStopCommand	PROC
;handle the stop command from the host
				LDAA			#RF_A
				STAA			requestFormat				;stop command forces format to absolute always, no matter what
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_DOOROPEN			;see if we should stop, or just ignore the command
				BEQ				..exit
				CMPA			#STATUS_TRAYMOVING
				BEQ				..exit
				LDAA			#MC_STOP					;tell the mechanism to stop
				JSR				CommandMechanism			;tell mechanism to spindown
				LDAA			#ES_STOPPING				;set us into the stopping state
				JSR				SetEmulationState
..exit:
				RTS
				ENDPROC


DoReportReqCommand	PROC
;the host wishes to change the report type, so try to do it
				LDAA			commandBuffer+CB_REQTYPE	;get the request type
				CMPA			#RF_A
				BNE				..notAbsolute
				STAA			requestFormat				;some requests just change the format, others request that the drive do things
				JMP				..exit
..notAbsolute:
				CMPA			#RF_R
				BNE				..notRelative
				STAA			requestFormat
				JMP				..exit
..notRelative:
				CMPA			#RF_T
				BNE				..notTrackInfo
				STAA			requestFormat
				JMP				..exit
..notTrackInfo:
				CMPA			#RF_TOCO
				BNE				..notTOCO

				JSR				DiscValidated				;see if the disc has been validated yet
				TSTA
				BNE				..ok1
				JMP				..error						;if disc not validated, then complain, do not change mode
..ok1:
				LDAA			#RF_TOCO					;get the type
				STAA			requestFormat				;this is the request format
				JMP				..exit
..notTOCO:
				CMPA			#RF_TOCT
				BNE				..notTOCT
				LDAB			driveStatus					;get current drive status
				CMPB			#STATUS_DOOROPEN			;if drive not ready, it is an error
				BEQ				..fail1
				CMPB			#STATUS_TRAYMOVING
				BNE				..ok2
..fail1:
				JMP				..error
..ok2:
				STAA			requestFormat				;this is the request format				
				JSR				DiscValidated
				TSTA
				BNE				..checkPauseInTOC			;disc is valid, so go pause in the TOC if the drive is stopped

				JSR				InitGetProgramArea
				JSR				InitTOCRead

				LDD				#0							;seek to starting location where we should find TOC info
				STD				m_seekAddress
				STD				m_seekAddress+2
				LDAA			#MC_SEEK					;begin seeking to the read location, go into play when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#MC_PLAY					;play when you get there
				JSR				CommandMechanism			;tell mechanism

				LDAA			#ES_TOCING
				JSR				SetEmulationState			;we are now trying to get the TOC
				BRA				..exit

..checkPauseInTOC:
				LDAA			driveStatus					;see if stopped
				CMPA			#STATUS_STOP				;if so, then go to pause in the TOC
				BNE				..exit						;if drive is not stopped, then just go into TOCT mode, do not halt current action
				LDD				#0							;seek to starting location where we should find TOC info
				STD				m_seekAddress
				STD				m_seekAddress+2
				LDAA			#MC_SEEK					;begin seeking to the location
				JSR				CommandMechanism			;tell mechanism
				LDAA			#MC_PAUSE					;pause when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_TOCING
				JSR				SetEmulationState			;we are now trying to get the TOC
				BRA				..exit

..notTOCT:
				CMPA			#RF_TOCN
				BNE				..notTOCN
				LDAB			driveStatus					;get current drive status
				CMPB			#STATUS_DOOROPEN			;if drive not ready, it is an error
				BEQ				..error
				CMPB			#STATUS_TRAYMOVING
				BEQ				..error
				CMPB			#STATUS_STOP
				BEQ				..error						;if drive is stopped, this is an error
				STAA			requestFormat
				LDAA			commandBuffer+CB_TOCNTNO	;get the high nibble of track number for TOCN
				ASLA										;shift up into place
				ASLA
				ASLA
				ASLA
				ORAA			commandBuffer+CB_TOCNTNO+1	;get low nibble into place
				JSR				BCDToBinary					;make into sane binary number
				STAA			TOCNTrackNumber				;keep it around for report
				LDD				#0							;seek to starting location where we should find TOC info
				STD				m_seekAddress
				STD				m_seekAddress+2
				LDAA			#MC_SEEK					;begin seeking to the location
				JSR				CommandMechanism			;tell mechanism
				LDAA			#MC_PAUSE					;pause when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_TOCING
				JSR				SetEmulationState			;we are now reading in the TOC
				BRA				..exit
..notTOCN:
				CMPA			#RF_E
				BNE				..error
				STAA			requestFormat
				BRA				..exit
..error:
;command is an error
				LDAA			#RF_A
				STAA			requestFormat				;on error, set format to RF_A
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoReadCommand	PROC
;read music, or rom data from the specified position
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_DOOROPEN			;see if we should read, or just ignore the command
				BEQ				..error
				CMPA			#STATUS_TRAYMOVING
				BEQ				..error
				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated
				TSTA
				BEQ				..error

				LDX				#m_seekAddress				;store block number from command here
				JSR				CommandTimeToBlock

				LDX				m_seekAddress				;see if requested block is out of range
				CPX				theTOC+TOC.leadoutStartBlock	;first, check the high word
				BHI				..error						;if high word of seek address is larger then high word of leadout start, it is an error
				BNE				..doSeek					;if it is lower, then the seek can proceed
				LDX				m_seekAddress+2				;see if requested block is out of range
				CPX				theTOC+TOC.leadoutStartBlock+2	;check the low word
				BCC				..error						;if high words were the same, and low of seek address is same, or larger than low of leadout, it is an error
..doSeek:
				LDX				#m_seekAddress				;convert to physical block number
				JSR				LBlockToPBlock
				LDX				#m_seekAddress				;prepare to step back a small amount
				JSR				LAdjustPBlockBack			;adjust the block that we are seeking to, back a small amount so that the desired block is not skipped accidentally
				LDAA			#MC_SEEK					;begin seeking to the read location, go into play when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#MC_PLAY					;play when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_READING
				JSR				SetEmulationState			;set the emulation state
				BRA				..exit
..error:
;read was issued when it should not have been, this is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoSeekCommand	PROC
;an attempt is being made to seek the drive, do it if possible
;the host wishes to read data, oblige him if possible
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_DOOROPEN			;see if we should seek, or just ignore the command
				BEQ				..error
				CMPA			#STATUS_TRAYMOVING
				BEQ				..error
				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated
				TSTA
				BEQ				..error

				LDX				#m_seekAddress				;store block number from command here
				JSR				CommandTimeToBlock

				LDX				m_seekAddress				;see if requested block is out of range
				CPX				theTOC+TOC.leadoutStartBlock	;first, check the high word
				BHI				..error						;if high word of seek address is larger then high word of leadout start, it is an error
				BNE				..doSeek					;if it is lower, then the seek can proceed
				LDX				m_seekAddress+2				;see if requested block is out of range
				CPX				theTOC+TOC.leadoutStartBlock+2	;check the low word
				BCC				..error						;if high words were the same, and low of seek address is same, or larger than low of leadout, it is an error
..doSeek:
				LDX				#m_seekAddress				;convert to physical block number
				JSR				LBlockToPBlock
				LDX				#m_seekAddress				;adjust seek block number
				JSR				LAdjustPBlockBack			;adjust the block that we are seeking to, back a small amount so that the desired block is not skipped accidentally
				LDAA			#MC_SEEK					;begin seeking to the location
				JSR				CommandMechanism			;tell mechanism
				LDAA			#MC_PAUSE					;pause when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_SEEKING
				JSR				SetEmulationState			;set the emulation state
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoPauseCommand	PROC
;drive is being paused
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_DOOROPEN			;see if we should pause, or just ignore the command
				BEQ				..error
				CMPA			#STATUS_TRAYMOVING
				BEQ				..error
				CMPA			#STATUS_SCAN
				BEQ				..error
				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated
				TSTA
				BEQ				..error

				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_STOP				;if stopped, we need to seek to start of disc and pause, else just pause
				BNE				..justPause

				LDX				#m_seekAddress				;destination
				LDAA			theTOC+TOC.firstTrack		;first track
				JSR				GetTrackPBlock				;get the physical block location for the first track
				LDAA			#MC_SEEK					;begin seeking to the location
				JSR				CommandMechanism			;tell mechanism
..justPause:
				LDAA			#MC_PAUSE					;pause when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_PAUSING
				JSR				SetEmulationState			;set the emulation state
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoPlayCommand	PROC
;play back from current position
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_DOOROPEN			;see if we should play, or just ignore the command
				BEQ				..error
				CMPA			#STATUS_TRAYMOVING
				BEQ				..error
				CMPA			#STATUS_DISCEND
				BEQ				..error
				CMPA			#STATUS_DISCIN
				BEQ				..error
				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated
				TSTA
				BEQ				..error

				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_TOCREAD				;if reading theTOC, but disc is valid, seek to start of first track and play
				BEQ				..seekAndPlay
				CMPA			#STATUS_STOP				;if stopped, we need to seek to start of disc and play, else just play
				BNE				..justPlay
..seekAndPlay:
				JSR				Recalibrate					;send the drive seeking to the start of the first track
..justPlay:
				LDAA			#MC_PLAY					;play when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_PLAYING
				JSR				SetEmulationState			;set the emulation state
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoFwdCommand	PROC
;skip forward
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_PLAY				;see if FWD command is allowed, and if so, start it
				BEQ				..doFwd
				CMPA			#STATUS_PAUSE
				BEQ				..doFwd
				CMPA			#STATUS_DISCIN
				BNE				..error						;if none of the above, then error
..doFwd:
				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated (just in case)
				TSTA
				BEQ				..error

				LDAA			#MC_PLAY					;play from where we are now
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_FWDING
				JSR				SetEmulationState			;set the emulation state
				LDAA			#SCANPLAYTIME
				STAA			scanClock					;initialize the scan clock
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoRvsCommand	PROC
;skip backward
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_PLAY				;see if FWD command is allowed, and if so, start it
				BEQ				..doRvs
				CMPA			#STATUS_PAUSE
				BEQ				..doRvs
				CMPA			#STATUS_DISCEND
				BNE				..error						;if none of the above, then error
..doRvs:
				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated (just in case)
				TSTA
				BEQ				..error

				LDAA			#MC_PLAY					;play from where we are now
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_RVSING
				JSR				SetEmulationState			;set the emulation state
				LDAA			#SCANPLAYTIME
				STAA			scanClock					;initialize the scan clock
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoTrackSkipCommand	PROC
;skip forward or backward a number of tracks
;the word track is not used in the conventional sense
;I believe it is meant to indicate roughly the number of arms of spiral data to step over
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_PLAY				;see if command is allowed, and if so, start it
				BEQ				..doSkip
				CMPA			#STATUS_PAUSE
				BEQ				..doSkip
				CMPA			#STATUS_DISCEND
				BNE				..error						;if none of the above, then error
..doSkip:
				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated
				TSTA
				BEQ				..error

				LDAA			commandBuffer+CB_DIRECTION	;get skip direction
				STAA			m_SkipDirection				;store it

				LDAA			commandBuffer+CB_NUMTRACKS	;get the highest nibble of number of tracks
				ASLA										;shift up into place
				ASLA
				ASLA
				ASLA
				ORAA			commandBuffer+CB_NUMTRACKS+1	;get next nibble into place
				STAA			m_SkipCount					;store high byte

				LDAA			commandBuffer+CB_NUMTRACKS+2	;get the second to last nibble of number of tracks
				ASLA										;shift up into place
				ASLA
				ASLA
				ASLA
				ORAA			commandBuffer+CB_NUMTRACKS+3	;get final nibble into place
				STAA			m_SkipCount+1				;store low byte

				LDAA			#MC_SKIP					;begin skipping
				JSR				CommandMechanism			;tell mechanism
				LDAA			#MC_PAUSE					;pause when you get there
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_TSKPING
				JSR				SetEmulationState			;set the emulation state
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoTrackCueCommand	PROC
;cue to a track
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_STOP				;we can cue from stop
				BEQ				..cueAndPlay
				CMPA			#STATUS_PLAY
				BEQ				..doCue
				CMPA			#STATUS_PAUSE
				BEQ				..doCue
				CMPA			#STATUS_DISCEND
				BEQ				..cueAndPlay
				CMPA			#STATUS_DISCIN
				BEQ				..cueAndPlay

				CMPA			#STATUS_TOCREAD
				BNE				..error						;if none of the above states, then command error

				JSR				ForceRequestFormat			;force the request format if needed
				JSR				DiscValidated				;see if the disc was validated
				TSTA
				BEQ				..error
..cueAndPlay:
				LDAA			#TRUE						;this tells if we should go into play after the seek
				PSHA
				BRA				..startCue
..doCue:
				LDAA			#FALSE
				PSHA
..startCue:
				LDAA			commandBuffer+CB_CUETNO		;get high nibble of track number
				ASLA										;shift into place
				ASLA										;shift into place
				ASLA										;shift into place
				ASLA										;shift into place
				ORAA			commandBuffer+CB_CUETNO+1	;add in low nibble
				JSR				BCDToBinary					;make into binary
				CMPA			theTOC+TOC.firstTrack		;see if lower than first track
				BCC				..notTooLow
				LDAA			theTOC+TOC.firstTrack		;if lower, make it the first track
..notTooLow:
				CMPA			theTOC+TOC.lastTrack		;see if higher than last track
				BLS				..notTooHigh
				LDAA			theTOC+TOC.lastTrack		;if higher, make it the last track
..notTooHigh:
				LDX				#m_seekAddress				;destination
				JSR				GetTrackPBlock				;get the physical block location for the track
				LDAA			#MC_SEEK					;begin seeking to the location
				JSR				CommandMechanism			;tell mechanism
				PULA										;see if we should go into play state, or leave old state
				TSTA
				BEQ				..dontPlay
				LDAA			#MC_PLAY					;play when you get there
				JSR				CommandMechanism			;tell mechanism
..dontPlay:
				LDAA			#ES_CUING
				JSR				SetEmulationState			;set the emulation state
				BRA				..exit
..error:
;read was issued when it should not have been, this is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoDoorCloseCommand	PROC
;close the door
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_DOOROPEN			;see if status is ok for command to execute
				BEQ				..doClose
				CMPA			#STATUS_TRAYMOVING
				BNE				..error

..doClose:
				LDAA			#MC_CLOSE
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_CLOSING
				JSR				SetEmulationState			;set the emulation state
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


DoDoorOpenCommand	PROC
;open the door
				LDAA			driveStatus					;get current drive status
				CMPA			#STATUS_DOOROPEN			;see if status is ok for command to execute
				BEQ				..error
				JSR				GetMechanismStatus			;see if drive must stop before door opens
				ANDA			#MS_SERVO					;isolate servo status
				CMPA			#MS_STOP
				BEQ				..justOpen
				LDAA			#RF_A
				STAA			requestFormat				;implicit stop command forces format to absolute
..justOpen:
				LDAA			#MC_OPEN
				JSR				CommandMechanism			;tell mechanism
				LDAA			#ES_OPENING
				JSR				SetEmulationState			;set the emulation state
				LDAA			#FALSE
				STAA			haveDiscValid				;once the door opens, we no longer know anything about the disc
				STAA			haveTOCValid
				STAA			havePAValid
				BRA				..exit
..error:
;command is an error
				LDAA			#STATUS_COMMANDERROR		;command error
				JSR				SetDriveErrorStatus
..exit:
				RTS
				ENDPROC


ProcessCommand	PROC
;After an attempt has been made to get a command from the host,
;call this to process it

				TST				statusSendFlag				;see if the status was sent properly
				BEQ				NOPCommand					;no, do nop, this does not change the current status
				TST				commandReceiveFlag			;see if a command was read in
				BEQ				NOPCommand					;no, just do nop

				TST				commandValidFlag			;see if we got a command, but it had bad checksum
				BNE				HandleCommand				;command was good, go handle it

				JSR				DoInvalidSum				;handle the bad checksum
				BRA				CommandProcessed
NOPCommand:
				JSR				DoNopCommand				;do the nop command
				BRA				CommandProcessed

;got command with valid checksum, so now handle it

HandleCommand:
				LDAA			commandBuffer+CB_COMMAND1	;get nibble 1 of command (must be zero)
				BEQ				Command1Ok
				JSR				DoInvalidCommand			;if low is not zero, call it invalid
				BRA				CommandProcessed
Command1Ok:
				LDAA			#TRUE
				STAA			statusOutputValid			;status output becomes valid on the receipt of a valid command
				JSR				PassCommandToIBM			;pass the command to the IBM
				LDX				#CommandVectorTable			;point to the command vector table
				LDAB			commandBuffer+CB_COMMAND0	;get nibble 0 of the command
				ASLB										;make into table index
				ABX											;index into the table
				LDX				0,X							;get the vector
				JSR				0,X							;go to the routine
CommandProcessed:
				RTS											;command has been processed

CommandVectorTable:
;### NOTE: the order of this table needs to be changed if the command
;numbers change!!!

				DW				DoNopCommand				;00
				DW				DoStopCommand				;10
				DW				DoReportReqCommand			;20
				DW				DoReadCommand				;30
				DW				DoSeekCommand				;40
				DW				DoInvalidCommand			;50
				DW				DoPauseCommand				;60
				DW				DoPlayCommand				;70
				DW				DoFwdCommand				;80
				DW				DoRvsCommand				;90
				DW				DoTrackSkipCommand			;A0
				DW				DoTrackCueCommand			;B0
				DW				DoDoorCloseCommand			;C0
				DW				DoDoorOpenCommand			;D0
				DW				DoInvalidCommand			;E0
				DW				DoInvalidCommand			;F0
				ENDPROC


MainLoop		PROC
;Main command loop

				JSR				GetSubcodeQ					;Get current subcode Q data
				JSR				ProcessCurrentState			;do the processing needed for the current state
				JSR				TalkToHost					;tell the host what is happening, and get new command
				JSR				ProcessCommand				;attempt to process any command that may have arrived from the host
				BRA				MainLoop

				ENDPROC


Main			PROC
;This is where it all happens
;Set everything up, then go into command loop

				JSR				InitHardwareForTest			;set up hardware so that it can be tested
				TSTA										;did hardware fail to init?
				BEQ				MainFail					;yes, time to die

				JSR				TestHardware				;see if system hardware is working
				TSTA										;did it fail?
				BEQ				MainFail					;yes, time to die
;
;System is functioning, setup for emulation
;
				JSR				InitHardwareForEmulation	;place hardware in the state needed for emulation
				TSTA										;failed to init?
				BEQ				MainFail					;yes, time to die

				JSR				InitGlobals					;make sure all my globals are set before interrupts are enabled
				TSTA										;failed to init?
				BEQ				MainFail					;yes, time to die

				JSR				ResetStatusBuffer			;set drive status to what it should be after a reset

				CLI											;allow all sorts of interruption
				JSR				DoReset						;reset the mechanism, and get into known state
				JMP				MainLoop					;begin the main loop 
MainFail:
;We have determined that we can no longer exist, so die
				SEI											;don't let interrupt wake us
				SLP											;go to sleep
				BRA				MainFail					;we woke, go back to sleep
				ENDPROC

;------------------------------------------------------------------------------------------------------------
;interrupt handling
;------------------------------------------------------------------------------------------------------------

IRQ1Vector		PROC
;IRQ1 interrupt handling code
;IRQ1 is caused by the bank select signal that occurs about every 13.3 ms
;It is also caused by the I2H stobe line that indicates that the IBM has
;sent us a byte

				LDAA			STAT_REG1					;see if bank IRQ needs to be handled
				ANDA			#SR1_mBANKIRQ
				CMPA			#SR1_aBANKIRQ
				BNE				ChkI2HIrq					;no, see if the IBM sent us a byte
				LDAA			ctrl2Shadow					;toggle bank IRQ clear/enable line
				EORA			#CR2_mBANKIRQ				;to clear the interrupt
				STAA			CTRL_REG2
				EORA			#CR2_mBANKIRQ
				STAA			CTRL_REG2

				INC				bankCounter					;bump the bank counter
				LDAA			scanClock					;see if we should count down scan clock
				BEQ				..noCount					;already down, dont count further
				DEC				scanClock					;count down scan clock
..noCount:
;###
				LDAA			LEDTimer					;make the debugging LED flash
				DECA
				STAA			LEDTimer
				BNE				DontToggle
				LDAA			ctrl1Shadow
				EORA			#CR1_mLED
				STAA			CTRL_REG1
				STAA			ctrl1Shadow
				LDAA			#75
				STAA			LEDTimer
DontToggle:
;###

ChkI2HIrq:
				LDAA			P6CSR						;get port control/status register
				ANDA			#ISFLAG						;see if input strobe happend
				BEQ				LeaveIRQ1					;no, leave
				LDAA			PORT6						;clear the interrupt
LeaveIRQ1:
				RTI
				ENDPROC


SIOVector		PROC
;Serial interrupt
;serial interrupts should not occur, because they are not used
;if they do, just ignore them

				RTI
				ENDPROC


IRQ2Vector		PROC
;IRQ2 interrupt
;IRQ2 is not used

				RTI
				ENDPROC


CMIVector		PROC
;timer 2 counter match interrupt

				RTI
				ENDPROC


TOIVector		PROC
;timer 1 overflow interrupt

				RTI
				ENDPROC


OCIVector		PROC
;output compare interrupt

				RTI
				ENDPROC


ICIVector		PROC
;timer1 input capture interrupt

				RTI
				ENDPROC


SWIVector		PROC
;software interrupt

				RTI
				ENDPROC


NMIVector		PROC
;NMI interrput
;An NMI occurs when the reset line from the host is activated
;It should cause us to reset, and start emulation all over again
;the reset line is filtered before it is applied to the NMI line, but
;it is wise to do some software de-glitching as well

				LDAB			#05							;must be active this many loops to be considered a reset
WaitForGlitch:
				LDAA			STAT_REG0					;get the state of the reset line
				ANDA			#SR0_mRESET					;mask it out
				CMPA			#SR0_iRESET					;did it go inactive?
				BEQ				NotReset					;yes, consider it a glitch, and ignore it
				DECB
				BNE				WaitForGlitch				;keep waiting to see if it is a glitch
				LDX				#stackBase					;restart system stack pointer
				TXS
				JMP				Main						;start running	
NotReset:
				RTI											;just leave, it was a glitch
				ENDPROC


TRAPVector		PROC
;TRAP interrupt

				RTI
				ENDPROC


ResetVector		PROC
;When the 6303 is reset, this is the first thing that happens

				LDX				#stackBase					;start system stack pointer quickly, so that NMI will not trash things
				TXS
				JMP				Main						;start running	
				ENDPROC


;Vectors for reset, and interrupts

				DEFSEG			IRQSeg,START=$FFEA			;this is where the vectors start
				SEG				IRQSeg

				DW				IRQ2Vector					;IRQ2 interrupt
				DW				CMIVector					;timer 2 counter match interrupt
				DW				TRAPVector					;:TRAP interrupt
				DW				SIOVector					;Serial interrupt
				DW				TOIVector					;timer 1 overflow interrupt
				DW				OCIVector					;output compare interrupt
				DW				ICIVector					;timer1 input capture interrupt
				DW				IRQ1Vector					;IRQ1 interrupt
				DW				SWIVector					;software interrupt
				DW				NMIVector					;NMI interrput
				DW				ResetVector					;Reset interrupt
				END
